# Generated by Django 4.1.10 on 2023-11-27 10:30

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0035_vectortilelayer_render_layers_json_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='vectortilelayer',
            name='popup_config',
            field=wagtail.fields.StreamField([('popup_fields', wagtail.blocks.StructBlock([('data_key', wagtail.blocks.CharBlock(label='Data Key', required=True)), ('label', wagtail.blocks.CharBlock(label='Popup Label', required=True)), ('data_type', wagtail.blocks.ChoiceBlock(choices=[('string', 'String'), ('number', 'Number')], label='Data Type'))], label='Popup Fields'))], blank=True, null=True, use_json_field=True, verbose_name='Map Popup Configuration'),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='current_time_method',
            field=models.CharField(choices=[('latest_from_source', 'Latest available date from source'), ('earliest_from_source', 'Earliest available date from source'), ('previous_to_now', 'Date previous to current date time'), ('next_to_now', 'Date next to current date time')], default='latest_from_source', help_text='How to pick default time and for updates, for Multi-Temporal data', max_length=100, verbose_name='current time method'),
        ),
    ]



# Generated by Django 4.2.1 on 2023-06-07 12:10

from django.conf import settings
import django.contrib.gis.db.models.fields
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_countries.fields
import django_extensions.db.fields
import geomanager.fields
import geomanager.models.raster_file
import modelcluster.fields
import uuid
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks
import wagtail_color_panel.blocks
import wagtail_color_panel.fields
import wagtailiconchooser.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailcore', '0083_workflowcontenttype'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('order', models.IntegerField(blank=True, editable=False, null=True)),
                ('title', models.CharField(help_text='Title of the category', max_length=16, verbose_name='title')),
                ('icon', models.CharField(blank=True, max_length=255, null=True, verbose_name='icon')),
                ('active', models.BooleanField(default=True, help_text='Is the category active ?', verbose_name='active')),
                ('public', models.BooleanField(default=True, help_text='Is the category public ?', verbose_name='public')),
            ],
            options={
                'verbose_name_plural': 'Categories',
                'ordering': ['order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CountryBoundary',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name_0', models.CharField(blank=True, max_length=100, null=True)),
                ('name_1', models.CharField(blank=True, max_length=100, null=True)),
                ('name_2', models.CharField(blank=True, max_length=100, null=True)),
                ('gid_0', models.CharField(blank=True, max_length=100, null=True)),
                ('gid_1', models.CharField(blank=True, max_length=100, null=True)),
                ('gid_2', models.CharField(blank=True, max_length=100, null=True)),
                ('level', models.IntegerField(blank=True, null=True)),
                ('size', models.CharField(blank=True, max_length=100, null=True)),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
            ],
            options={
                'verbose_name_plural': 'Country Boundaries',
            },
        ),
        migrations.CreateModel(
            name='Dataset',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='The Dataset title as will appear to the public', max_length=255, verbose_name='title')),
                ('summary', models.CharField(blank=True, help_text='Short summary of less than 100 characters', max_length=100, null=True, verbose_name='summary')),
                ('layer_type', models.CharField(choices=[('file', 'Raster'), ('wms', 'WMS'), ('vector', 'Vector')], default='file', max_length=100, verbose_name='layer_type')),
                ('published', models.BooleanField(default=True, help_text='Should the dataset be available for visualization ? If unchecked, the dataset is assumed to be in draft mode and thus not ready', verbose_name='published')),
                ('public', models.BooleanField(default=True, help_text='Should the dataset be visible to everyone ? If unchecked, only authorized users can view', verbose_name='public')),
                ('initial_visible', models.BooleanField(default=False, help_text='Make the dataset visible on the map by default', verbose_name='Initially visible on Map by default')),
                ('multi_temporal', models.BooleanField(default=True, help_text='The dataset is multi-temporal', verbose_name='multi-temporal')),
                ('multi_layer', models.BooleanField(default=False, help_text='The dataset has more than one layer, to be displayed together', verbose_name='multi-layer')),
                ('near_realtime', models.BooleanField(default=False, help_text='Is the layer near realtime?, for example updates every 10 minutes', verbose_name='near realtime')),
                ('current_time_method', models.CharField(choices=[('latest_from_source', 'Latest From Source'), ('previous_to_now', 'Previous to now'), ('next_to_now', 'Next to Now')], default='latest_from_source', help_text='How to pick default time and for updates, for Multi-Temporal data', max_length=100, verbose_name='current time method')),
                ('auto_update_interval', models.IntegerField(blank=True, help_text='After how many minutes should the layer auto update on the map to show current data, if multi-temporal. Leave empty to disable auto updating', null=True, verbose_name='Auto Update interval in minutes')),
                ('can_clip', models.BooleanField(default=False, help_text='Check to enable clipping by boundary or drawn shapes, for raster and vector datasets. Not implemented for WMS types', verbose_name='Enable Clipping by shape')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='geomanager.category', verbose_name='category')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Geostore',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('iso', models.CharField(blank=True, max_length=100, null=True)),
                ('id1', models.CharField(blank=True, max_length=100, null=True)),
                ('id2', models.CharField(blank=True, max_length=100, null=True)),
                ('name_0', models.CharField(blank=True, max_length=100, null=True)),
                ('name_1', models.CharField(blank=True, max_length=100, null=True)),
                ('name_2', models.CharField(blank=True, max_length=100, null=True)),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MBTSource',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(help_text='Style Name', max_length=255, verbose_name='name')),
                ('slug', models.CharField(editable=False, max_length=255, unique=True)),
                ('file', models.FileField(upload_to='mbtiles', verbose_name='file')),
                ('use_default_style', models.BooleanField(default=True, verbose_name='Use default style')),
                ('open_map_style_json', models.JSONField(blank=True, help_text="OpenMapTiles JSON Style. See schema here: <a href='https://openmaptiles.org/schema' target='_blank' rel='noopener noreferrer'>https://openmaptiles.org/schema</a>", null=True, verbose_name='OpenMapTiles style')),
            ],
            options={
                'verbose_name': 'Basemap Source',
            },
        ),
        migrations.CreateModel(
            name='Metadata',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='Title of the dataset', max_length=255, verbose_name='title')),
                ('subtitle', models.CharField(blank=True, help_text='Subtitle if any', max_length=255, null=True, verbose_name='subtitle')),
                ('function', models.TextField(blank=True, help_text='Short summary of what the dataset shows. Keep it short.', max_length=255, null=True, verbose_name='Dataset summary')),
                ('resolution', models.CharField(blank=True, help_text='The Spatial resolution of the dataset, for example 10km by 10 km', max_length=255, null=True, verbose_name='resolution')),
                ('geographic_coverage', models.CharField(blank=True, help_text='The geographic coverage of the dataset. For example East Africa or specific country name like Ethiopia, or Africa', max_length=255, null=True, verbose_name='Geographic coverage')),
                ('source', models.CharField(blank=True, help_text='The source of the data where was it generated or produced', max_length=255, null=True, verbose_name='source')),
                ('license', models.CharField(blank=True, help_text='Any licensing information for the dataset', max_length=255, null=True, verbose_name='license')),
                ('frequency_of_update', models.CharField(blank=True, help_text='How frequent is the dataset updated. For example daily, weekly, monthly etc', max_length=255, null=True, verbose_name='Frequency of updates')),
                ('overview', wagtail.fields.RichTextField(blank=True, help_text='Detail description of the dataset, including the methodology, references or any other relevant information', null=True, verbose_name='detail')),
                ('cautions', wagtail.fields.RichTextField(blank=True, help_text='What things should users be aware as they use and interpret this dataset', null=True, verbose_name='cautions')),
                ('citation', wagtail.fields.RichTextField(blank=True, help_text='Scientific citation for this dataset if any. For example the citation for a scientific paper for the dataset', null=True, verbose_name='citation')),
                ('download_data', models.URLField(blank=True, help_text='External link to where the source data can be found and downloaded', null=True, verbose_name='Data download link')),
                ('learn_more', models.URLField(blank=True, help_text='External link to where more detail about the dataset can be found', null=True, verbose_name='Learn more link')),
            ],
            options={
                'verbose_name_plural': 'Metadata',
            },
        ),
        migrations.CreateModel(
            name='RasterStyle',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(help_text='Style name for identification', max_length=256, verbose_name='name')),
                ('unit', models.CharField(blank=True, help_text='Data unit', max_length=100, null=True, verbose_name='data unit')),
                ('min', models.IntegerField(default=0, help_text='minimum value', verbose_name='minimum value')),
                ('max', models.IntegerField(default=100, help_text='maximum value', verbose_name='maximum value')),
                ('steps', models.IntegerField(blank=True, default=5, help_text='Number of steps', null=True, validators=[django.core.validators.MinValueValidator(3), django.core.validators.MaxValueValidator(20)], verbose_name='steps')),
                ('use_custom_colors', models.BooleanField(default=False, verbose_name='Use Custom Colors')),
                ('palette', models.TextField(blank=True, null=True, verbose_name='Color Palette')),
                ('interpolate', models.BooleanField(default=False, help_text='Interpolate colorscale', verbose_name='interpolate')),
                ('custom_color_for_rest', wagtail_color_panel.fields.ColorField(blank=True, default='#ff0000', help_text='Color for values greater than the values defined above, as well as values greater than the maximum value', max_length=7, null=True, verbose_name='Color for the rest of values')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WmsLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('base_url', models.CharField(max_length=500, verbose_name='base url for WMS')),
                ('version', models.CharField(choices=[('1.1.1', '1.1.1'), ('1.3.0', '1.3.0')], default='1.1.1', max_length=50, verbose_name='WMS Version')),
                ('width', models.IntegerField(default=256, help_text='The size of the map image in pixels along the i axis', verbose_name='Pixel Width')),
                ('height', models.IntegerField(default=256, help_text='The size of the map image in pixels along the j axis', verbose_name='Pixel Height')),
                ('transparent', models.BooleanField(default=True, help_text='Ability of underlying maps to be visible or not', verbose_name='Transparency')),
                ('srs', models.CharField(default='EPSG:3857', help_text='WMS Spatial Reference e.g EPSG:3857', max_length=50, verbose_name='Spatial Reference System')),
                ('format', models.CharField(choices=[('image/png', 'PNG'), ('image/jpeg', 'JPEG'), ('image/gif', 'GIF')], default='image/png', help_text='Allowed map formats are either “picture” formats or “graphic element” formats.', max_length=50, verbose_name='Output Format')),
                ('wms_query_params_selectable', wagtail.fields.StreamField([('param', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock(label='name')), ('label', wagtail.blocks.CharBlock(label='label', required=False)), ('type', wagtail.blocks.ChoiceBlock(choices=[('radio', 'Radio'), ('dropdown', 'Dropdown')], label='Selector Type')), ('options', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(label='label')), ('value', wagtail.blocks.CharBlock(label='value')), ('default', wagtail.blocks.BooleanBlock(help_text='Check to make default option', label='default', required=False))]), label='Options', min_num=1))], label='Query Parameter'))], blank=True, help_text='This should provide a list of options that users can choose to change the query parameter of the url', null=True, use_json_field=True, verbose_name='WMS Query Params With selectable Options')),
                ('params_selectors_side_by_side', models.BooleanField(default=False, verbose_name='Arrange Param Selectors side by side')),
                ('legend', wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Custom Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Custom Image'))], blank=True, null=True, use_json_field=True, verbose_name='Legend')),
                ('dataset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='wms_layers', to='geomanager.dataset', verbose_name='dataset')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WmsRequestStyle',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.CharField(help_text="The style's Name is used in the Map request STYLES parameter", max_length=250, verbose_name='name')),
                ('layer', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='wms_request_styles', to='geomanager.wmslayer', verbose_name='WMS Request Styles')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WmsRequestParam',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.CharField(help_text='Name of the parameter', max_length=250, verbose_name='name')),
                ('value', models.CharField(help_text='Value of the parameter', max_length=250)),
                ('layer', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='wms_request_params', to='geomanager.wmslayer', verbose_name='WMS Requests Additional Parameters')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='WmsRequestLayer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.CharField(help_text='WMS Layer is requested by using this name in the LAYERS parameter of a GetMap request.', max_length=250, verbose_name='name')),
                ('layer', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='wms_request_layers', to='geomanager.wmslayer', verbose_name='WMS Request Layers')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VectorUpload',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('file', models.FileField(upload_to='vector_uploads')),
                ('vector_metadata', models.JSONField(blank=True, null=True)),
                ('dataset', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.dataset')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VectorLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('render_layers', wagtail.fields.StreamField([('fill', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('fill_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='fill color', required=False)), ('fill_opacity', wagtail.blocks.FloatBlock(default=1.0, label='fill opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('fill_outline_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='fill outline color', required=False)), ('fill_antialias', wagtail.blocks.BooleanBlock(default=True, label='fill antialias', required=False))], label='Paint Properties')), ('filter', wagtail.blocks.CharBlock(label='filter', required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(label='maxzoom', required=False)), ('minzoom', wagtail.blocks.IntegerBlock(label='minzoom', required=False))], label='Polygon Layer')), ('line', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('line_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Line color', required=False)), ('line_dasharray', wagtail.blocks.CharBlock(label='Line dasharray', required=False)), ('line_gap_width', wagtail.blocks.FloatBlock(default=0.0, label='Line gap width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_opacity', wagtail.blocks.FloatBlock(default=1.0, label='line opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('line_width', wagtail.blocks.FloatBlock(default=1.0, label='Line width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_offset', wagtail.blocks.FloatBlock(default=0, label='Line offset', required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Paint Properties')), ('layout', wagtail.blocks.StructBlock([('line_cap', wagtail.blocks.ChoiceBlock(choices=[('butt', 'Butt'), ('round', 'Round'), ('square', 'Square')], label='Line cap', required=False)), ('line_join', wagtail.blocks.ChoiceBlock(choices=[('miter', 'Miter'), ('bevel', 'Bevel'), ('round', 'Round')], label='Line join', required=False)), ('line_miter_limit', wagtail.blocks.FloatBlock(default=2.0, label='line miter limit', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_round_limit', wagtail.blocks.FloatBlock(default=1.05, label='line round limit', required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Layout Properties')), ('filter', wagtail.blocks.CharBlock(label='filter', required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(label='maxzoom', required=False)), ('minzoom', wagtail.blocks.IntegerBlock(label='minzoom', required=False))], label='Line Layer')), ('circle', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('circle_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='circle color', required=False)), ('circle_opacity', wagtail.blocks.FloatBlock(default=1.0, label='circle opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('circle_radius', wagtail.blocks.FloatBlock(default=5.0, label='circle radius', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('circle_stroke_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='circle stroke color', required=False)), ('circle_stroke_width', wagtail.blocks.FloatBlock(default=0.0, label='circle_stroke_width', required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Paint Properties')), ('filter', wagtail.blocks.CharBlock(label='filter', required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(label='maxzoom', required=False)), ('minzoom', wagtail.blocks.IntegerBlock(label='minzoom', required=False))], label='Point Layer')), ('icon', wagtail.blocks.StructBlock([('layout', wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_allow_overlap', wagtail.blocks.BooleanBlock(default=False, label='Icon allow overlap', required=False)), ('icon_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], label='Icon anchor', required=False)), ('icon_ignore_placement', wagtail.blocks.BooleanBlock(default=False, label='Icon ignore placement', required=False)), ('icon_keep_upright', wagtail.blocks.BooleanBlock(default=False, label='Icon keep upright', required=False)), ('icon_offset', wagtail.blocks.CharBlock(label='Icon offset', required=False)), ('icon_optional', wagtail.blocks.BooleanBlock(default=False, label='Icon optional', required=False)), ('icon_padding', wagtail.blocks.FloatBlock(default=2.0, label='Icon padding', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Icon pitch alignment', required=False)), ('icon_rotate', wagtail.blocks.IntegerBlock(default=0, label='icon rotate', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('icon_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Icon rotation alignment', required=False)), ('icon_size', wagtail.blocks.FloatBlock(default=1, label='icon size', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('icon_text_fit', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('width', 'Width'), ('height', 'Height'), ('both', 'Both')], label='Icon text fit', required=False))], label='Layout Properties')), ('paint', wagtail.blocks.StructBlock([('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon color', required=False)), ('icon_halo_blur', wagtail.blocks.FloatBlock(default=0.0, label='Icon halo blur', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon halo color', required=False)), ('icon_halo_width', wagtail.blocks.FloatBlock(default=0.0, label='icon halo width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('icon_opacity', wagtail.blocks.FloatBlock(default=1.0, label='icon opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)]))], label='Paint Properties'))], label='Icon Layer')), ('text', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('text_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Text color', required=False)), ('text_halo_blur', wagtail.blocks.FloatBlock(default=0.0, label='Text halo blur', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_halo_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Text halo color', required=False)), ('text_halo_width', wagtail.blocks.FloatBlock(default=0.0, label='text halo width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_translate', wagtail.blocks.CharBlock(label='Text translate', required=False)), ('text_translate_anchor', wagtail.blocks.ChoiceBlock(choices=[('map', 'Map'), ('viewport', 'Viewport')], label='Text translate anchor', required=False))], label='Paint Properties')), ('layout', wagtail.blocks.StructBlock([('symbol_placement', wagtail.blocks.ChoiceBlock(choices=[('point', 'Point'), ('line', 'Line'), ('line-center', 'Line Center')], label='Text Placement', required=False)), ('text_allow_overlap', wagtail.blocks.BooleanBlock(default=False, label='Text allow overlap', required=False)), ('text_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], label='Text anchor', required=False)), ('text_field', wagtail.blocks.CharBlock(label='Text field')), ('text_size', wagtail.blocks.IntegerBlock(default=16, label='Text size', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_transform', wagtail.blocks.ChoiceBlock(choices=[('none', 'None'), ('uppercase', 'Uppercase'), ('lowercase', 'Lowercase')], label='Text transform', required=False)), ('text_ignore_placement', wagtail.blocks.BooleanBlock(default=False, label='Text ignore placement', required=False)), ('text_justify', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('auto', 'Auto')], label='Text justify', required=False)), ('text_keep_upright', wagtail.blocks.BooleanBlock(default=False, label='Text keep upright', required=False)), ('text_letter_spacing', wagtail.blocks.FloatBlock(default=0.0, label='Text letter spacing', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_line_height', wagtail.blocks.FloatBlock(default=1.2, label='Text line height', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_max_angle', wagtail.blocks.IntegerBlock(default=45, label='Text max angle', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_max_width', wagtail.blocks.IntegerBlock(default=10, label='Text max width', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_offset', wagtail.blocks.CharBlock(label='Text offset', required=False)), ('text_opacity', wagtail.blocks.FloatBlock(default=1.0, label='text opacity', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('text_padding', wagtail.blocks.IntegerBlock(default=2, label='Text  padding', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_pitch_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Text pitch alignment', required=False)), ('text_radial_offset', wagtail.blocks.IntegerBlock(default=0, label='Text radial offset', required=False, validators=[django.core.validators.MinValueValidator(0)])), ('text_rotate', wagtail.blocks.IntegerBlock(default=0, label='Text rotate', required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(360)])), ('text_rotation_alignment', wagtail.blocks.ChoiceBlock(choices=[('auto', 'Auto'), ('map', 'Map'), ('viewport', 'Viewport')], label='Text rotation alignment', required=False)), ('text_variable_anchor', wagtail.blocks.ChoiceBlock(choices=[('center', 'Center'), ('left', 'Left'), ('right', 'Right'), ('top', 'Top'), ('bottom', 'Bottom'), ('top-left', 'Top Left'), ('top-right', 'Top Right'), ('bottom-left', 'Bottom Left'), ('bottom-right', 'Bottom Right')], label='Text variable anchor', required=False))], label='Layout Properties'))], label='Text Label Layer'))], blank=True, null=True, use_json_field=True, verbose_name='Render Layers')),
                ('legend', wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Legend Image')), ('legend_icon', wagtail.blocks.StructBlock([('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_label', wagtail.blocks.CharBlock(label='Icon Label')), ('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon color', required=False))]), label='Legend Icons', min_num=1))], label='Legend Icon'))], blank=True, null=True, use_json_field=True, verbose_name='Legend')),
                ('dataset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='vector_layers', to='geomanager.dataset', verbose_name='dataset')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SubCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('title', models.CharField(max_length=256, verbose_name='title')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
                ('public', models.BooleanField(default=True, verbose_name='public')),
                ('category', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='sub_categories', to='geomanager.category')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RasterUpload',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('file', models.FileField(upload_to='raster_uploads', verbose_name='file')),
                ('raster_metadata', models.JSONField(blank=True, null=True)),
                ('dataset', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.dataset', verbose_name='dataset')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeoManagerUserProfile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('gender', models.CharField(max_length=50, verbose_name='Gender')),
                ('country', models.CharField(max_length=50, verbose_name='Country')),
                ('city', models.CharField(max_length=255, verbose_name='City')),
                ('sector', models.TextField(verbose_name='Sector')),
                ('organization', models.CharField(max_length=255, verbose_name='Organization')),
                ('organization_type', models.TextField(max_length=255, verbose_name='Organization Type')),
                ('scale_of_operations', models.CharField(max_length=255, verbose_name='Scale of Operations')),
                ('position', models.CharField(max_length=255, verbose_name='Position')),
                ('how_do_you_use', models.CharField(blank=True, max_length=255, null=True, verbose_name='How do you Use')),
                ('interests', models.TextField(blank=True, null=True, verbose_name='Interests')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeomanagerSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('max_upload_size_mb', models.IntegerField(default=10, help_text='Maximum raster file size that can be uploaded in MegaBytes. Default is 10Mbs.', verbose_name='Maximum upload size in MegaBytes')),
                ('cap_base_url', models.URLField(blank=True, max_length=256, null=True, verbose_name='cap base url')),
                ('cap_auto_refresh_interval', models.IntegerField(blank=True, help_text='Refresh cap alerts on the map after this minutes. Leave blank to disable auto refreshing', null=True, verbose_name='Auto Refresh interval in minutes')),
                ('cap_shown_by_default', models.BooleanField(default=True, help_text='CAP Layer shown on map by default', verbose_name='CAP layer shown by default')),
                ('country', django_countries.fields.CountryField(max_length=2, verbose_name='country')),
                ('gadm_version', models.CharField(choices=[('4.1', '4.1')], default='4.1', help_text='It is encouraged to use the latest version. See available versions here: https://gadm.org/old_versions.html', max_length=50, verbose_name='GADM Boundary Data Version')),
                ('tile_gl_fonts_url', models.URLField(default='https://fonts.openmaptiles.org/{fontstack}/{range}.pbf', help_text='GL Styles Font Url', max_length=256, verbose_name='GL Styles Font Url')),
                ('base_maps', wagtail.fields.StreamField([('basemap', wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(label='label')), ('backgroundColor', wagtail.blocks.CharBlock(label='background color')), ('image', wagtail.images.blocks.ImageChooserBlock(label='image', required=False)), ('basemapGroup', wagtail.blocks.CharBlock(label='basemap group')), ('labelsGroup', wagtail.blocks.CharBlock(label='labels group')), ('url', wagtail.blocks.URLBlock(label='url', required=False)), ('default', wagtail.blocks.BooleanBlock(help_text='Is default style ?', label='default', required=False))]))], blank=True, null=True, use_json_field=True)),
                ('cap_metadata', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.metadata', verbose_name='Metadata')),
                ('cap_sub_category', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.subcategory', verbose_name='cap layer sub category')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
                ('tile_gl_source', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.mbtsource', verbose_name='Open Map Tiles Source')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FileImageLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('analysis', wagtail.fields.StreamField([('point_analysis', wagtail.blocks.StructBlock([('instance_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show data for point', required=False)), ('timeseries_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show timeseries data for point', required=False)), ('unit', wagtail.blocks.CharBlock(label='Data unit', required=False)), ('timeseries_chart_type', wagtail.blocks.ChoiceBlock(choices=[('lines', 'Line Chart'), ('bars', 'Bar Chart')], label='Chart Type')), ('timeseries_chart_color', wagtail_color_panel.blocks.NativeColorBlock(default='#367DA4', label='Chart Line/Bar Color', required=True))], label='Point Analysis')), ('area_analysis', wagtail.blocks.StructBlock([('instance_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show data for area', required=False)), ('instance_value_type', wagtail.blocks.ChoiceBlock(choices=[('mean', 'Mean of pixel values'), ('sum', 'Sum of pixel values'), ('minmax', 'Minimum, Maximum pixel values'), ('minmeanmax', 'Minimum, Mean, Maximum pixel values')], help_text='The value type that should be displayed', label='Area value type')), ('unit', wagtail.blocks.CharBlock(label='Data unit', required=False)), ('timeseries_data_enabled', wagtail.blocks.BooleanBlock(default=True, label='Show timeseries data for area', required=False)), ('timeseries_aggregation_method', wagtail.blocks.ChoiceBlock(choices=[('mean', 'By Mean'), ('sum', 'By Sum')], help_text='How should the region data be aggregated ?', label='Area timeseries data aggregation Method')), ('timeseries_chart_type', wagtail.blocks.ChoiceBlock(choices=[('lines', 'Line Chart'), ('bars', 'Bar Chart')], label='Chart Type')), ('timeseries_chart_color', wagtail_color_panel.blocks.NativeColorBlock(default='#367DA4', label='circle stroke color', required=True))], label='Area Analysis'))], blank=True, null=True, use_json_field=True, verbose_name='Analysis')),
                ('dataset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='file_layers', to='geomanager.dataset', verbose_name='dataset')),
                ('style', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.rasterstyle', verbose_name='style')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='dataset',
            name='metadata',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.metadata', verbose_name='metadata'),
        ),
        migrations.AddField(
            model_name='dataset',
            name='sub_category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='geomanager.subcategory', verbose_name='sub_category'),
        ),
        migrations.CreateModel(
            name='ColorValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('threshold', models.FloatField(verbose_name='threshold')),
                ('color', wagtail_color_panel.fields.ColorField(default='#ff0000', max_length=7, verbose_name='color')),
                ('label', models.CharField(blank=True, max_length=100, null=True, verbose_name='label')),
                ('layer', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='color_values', to='geomanager.rasterstyle')),
            ],
            options={
                'verbose_name': 'Color Value',
                'verbose_name_plural': 'Color Values',
            },
        ),
        migrations.CreateModel(
            name='AreaOfInterest',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('adm_0', models.CharField(blank=True, max_length=255, null=True, verbose_name='Adm 0')),
                ('adm_1', models.CharField(blank=True, max_length=255, null=True, verbose_name='Adm 1')),
                ('adm_2', models.CharField(blank=True, max_length=255, null=True, verbose_name='Adm 2')),
                ('adm_3', models.CharField(blank=True, max_length=255, null=True, verbose_name='Adm 3')),
                ('public', models.BooleanField(default=False, verbose_name='Public')),
                ('type', models.CharField(blank=True, max_length=255, null=True, verbose_name='Type')),
                ('products', models.TextField(blank=True, null=True, verbose_name='Products')),
                ('tags', models.TextField(blank=True, null=True, verbose_name='Tags')),
                ('webhook_url', models.URLField(blank=True, null=True, verbose_name='Webhook Url')),
                ('geostore_id', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='geomanager.geostore', verbose_name='Geostore Id')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PgVectorTable',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('table_name', models.CharField(max_length=256, unique=True)),
                ('full_table_name', models.CharField(max_length=256)),
                ('description', models.TextField(blank=True, null=True)),
                ('time', models.DateTimeField(help_text='time for the dataset')),
                ('properties', models.JSONField()),
                ('geometry_type', models.CharField(max_length=100)),
                ('bounds', geomanager.fields.ListField(max_length=256)),
                ('layer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='vector_tables', to='geomanager.vectorlayer')),
            ],
            options={
                'ordering': ['-time'],
                'unique_together': {('layer', 'time')},
            },
        ),
        migrations.CreateModel(
            name='LayerRasterFile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('file', models.FileField(editable=False, upload_to=geomanager.models.raster_file.layer_raster_file_dir_path, verbose_name='file')),
                ('time', models.DateTimeField(help_text='Time for the raster file. This can be the time the data was acquired, or the date and time for which the data applies', verbose_name='time')),
                ('layer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='raster_files', to='geomanager.fileimagelayer', verbose_name='layer')),
            ],
            options={
                'ordering': ['time'],
                'unique_together': {('layer', 'time')},
            },
        ),
    ]



# Generated by Django 4.1.10 on 2023-10-27 08:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0026_layerrasterfile_raster_metadata_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='layerrasterfile',
            options={'ordering': ['-time'], 'verbose_name': 'Layer Raster File', 'verbose_name_plural': 'Layer Raster Files'},
        ),
        migrations.AddField(
            model_name='rasterfilelayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)')], default='yyyy-MM-dd HH:mm', max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
        migrations.AddField(
            model_name='rastertilelayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)')], max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
        migrations.AddField(
            model_name='vectortilelayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)')], max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
        migrations.AlterField(
            model_name='wmslayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)')], max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
    ]



from rest_framework import mixins, viewsets
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from wagtail import hooks

from geomanager import serializers
from geomanager.models import Dataset
from geomanager.models.core import Metadata

geomanager_register_datasets_hook_name = "register_geomanager_datasets"


class DatasetViewSet(mixins.ListModelMixin, mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    queryset = Dataset.objects.filter(published=True)
    serializer_class = serializers.DatasetSerializer

    renderer_classes = [JSONRenderer]

    def get_serializer_context(self):
        context = super().get_serializer_context()
        context.update({'request': self.request})
        return context

    def list(self, request, *args, **kwargs):
        queryset = self.get_queryset()
        dataset_with_layers = []

        # get only datasets with layers defined
        for dataset in queryset:
            if dataset.has_layers():
                dataset_with_layers.append(dataset)

        serializer = self.get_serializer(dataset_with_layers, many=True)

        datasets = serializer.data

        # get datasets from registered hooks
        for fn in hooks.get_hooks(geomanager_register_datasets_hook_name):
            hook_datasets = fn(request)
            for dataset in hook_datasets:
                datasets.append(dataset)

        return Response(datasets)


class MetadataViewSet(mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    queryset = Metadata.objects.all()
    serializer_class = serializers.MetadataSerialiazer



from rest_framework import serializers

from geomanager.models import LayerRasterFile, RasterFileLayer


class RasterFileLayerSerializer(serializers.ModelSerializer):
    layerConfig = serializers.SerializerMethodField()
    params = serializers.SerializerMethodField()
    paramsSelectorConfig = serializers.SerializerMethodField()
    legendConfig = serializers.SerializerMethodField()
    name = serializers.SerializerMethodField()
    layerType = serializers.SerializerMethodField()
    multiTemporal = serializers.SerializerMethodField()
    currentTimeMethod = serializers.SerializerMethodField()
    autoUpdateInterval = serializers.SerializerMethodField()
    isMultiLayer = serializers.SerializerMethodField()
    nestedLegend = serializers.SerializerMethodField()
    canClip = serializers.SerializerMethodField()
    analysisConfig = serializers.SerializerMethodField()
    tileJsonUrl = serializers.SerializerMethodField()

    class Meta:
        model = RasterFileLayer
        fields = ["id", "dataset", "name", "layerType", "multiTemporal", "isMultiLayer", "legendConfig", "nestedLegend",
                  "layerConfig", "params", "paramsSelectorConfig", "currentTimeMethod", "autoUpdateInterval", "canClip",
                  "analysisConfig", "tileJsonUrl"]

    def get_isMultiLayer(self, obj):
        return obj.dataset.multi_layer

    def get_nestedLegend(self, obj):
        return obj.dataset.multi_layer

    def get_autoUpdateInterval(self, obj):
        return obj.dataset.auto_update_interval_milliseconds

    def get_multiTemporal(self, obj):
        return obj.dataset.multi_temporal

    def get_layerType(self, obj):
        return obj.dataset.layer_type

    def get_name(self, obj):
        return obj.title

    def get_layerConfig(self, obj):
        request = self.context.get('request')
        layer_config = obj.layer_config(request)
        return layer_config

    def get_tileJsonUrl(self, obj):
        request = self.context.get('request')
        tile_json_url = obj.get_tile_json_url(request)
        return tile_json_url

    def get_params(self, obj):
        return obj.params

    def get_paramsSelectorConfig(self, obj):
        return obj.param_selector_config

    def get_legendConfig(self, obj):
        return obj.get_legend_config()

    def get_currentTimeMethod(self, obj):
        return obj.dataset.current_time_method

    def get_canClip(self, obj):
        return obj.dataset.can_clip

    def get_analysisConfig(self, obj):
        return obj.get_analysis_config()


class LayerRasterFileSerializer(serializers.ModelSerializer):
    time = serializers.SerializerMethodField()

    class Meta:
        model = LayerRasterFile
        fields = ["time", "id"]

    def get_time(self, obj):
        return obj.time.strftime("%Y-%m-%dT%H:%M:%S.000Z")



from django.urls import path
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail_modeladmin.helpers import AdminURLHelper, ButtonHelper
from wagtail_modeladmin.views import CreateView, EditView, IndexView

from geomanager.admin.base import BaseModelAdmin, ModelAdminCanHide, LayerIndexView, LayerFileDeleteView
from geomanager.models import Dataset, RasterFileLayer, LayerRasterFile, Category
from geomanager.views import (
    upload_raster_file,
    publish_raster,
    delete_raster_upload,
    preview_raster_layers
)


class RasterFileLayerCreateView(CreateView):
    form_view_extra_js = ["geomanager/js/raster-file-conditional.js"]

    def get_form(self):
        form = super().get_form()
        form.fields["dataset"].queryset = Dataset.objects.filter(layer_type="raster_file")

        dataset_id = self.request.GET.get("dataset_id")
        if dataset_id:
            initial = {**form.initial}
            initial.update({"dataset": dataset_id})
            form.initial = initial
        return form

    def get_context_data(self, **kwargs):
        context_data = super(RasterFileLayerCreateView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": "#", "label": _("New") + f" {RasterFileLayer._meta.verbose_name}"},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class RasterFileLayerEditView(EditView):
    def get_form(self):
        form = super().get_form()
        form.fields["dataset"].queryset = Dataset.objects.filter(layer_type="raster_file")
        return form

    def get_context_data(self, **kwargs):
        context_data = super(RasterFileLayerEditView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        layer_admin_helper = AdminURLHelper(RasterFileLayer)
        layer_index_url = layer_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": layer_index_url, "label": RasterFileLayer._meta.verbose_name_plural},
            {"url": "#", "label": self.instance.title},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class RasterFileLayerButtonHelper(ButtonHelper):
    def get_buttons_for_obj(
            self, obj, exclude=None, classnames_add=None, classnames_exclude=None
    ):
        buttons = super().get_buttons_for_obj(obj, exclude, classnames_add, classnames_exclude)

        classnames = self.edit_button_classnames + classnames_add
        cn = self.finalise_classname(classnames, classnames_exclude)

        url = obj.get_style_url()

        layer_style_button = {
            "url": url.get("url"),
            "label": _(url.get("action")),
            "classname": cn,
            "title": _(url.get("action")) % {"object": self.verbose_name},
        }

        buttons.append(layer_style_button)

        return buttons


class RasterFileLayerModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = RasterFileLayer
    hidden = True
    exclude_from_explorer = True
    menu_label = _("File Layers")
    button_helper_class = RasterFileLayerButtonHelper
    list_display = ("title",)
    list_filter = ("dataset",)
    index_template_name = "geomanager/modeladmin/index_without_custom_create.html"
    list_display_add_buttons = "title"

    index_view_class = LayerIndexView
    create_view_class = RasterFileLayerCreateView
    edit_view_class = RasterFileLayerEditView

    form_view_extra_js = ["geomanager/js/raster-file-conditional.js"]

    def __init__(self, parent=None):
        super().__init__(parent)
        self.list_display = ["thumbnail_url"] + (list(self.list_display) or []) + ["dataset_link", "upload_files",
                                                                                   "uploaded_files", "preview_layer",
                                                                                   "mapviewer_map_url"]
        self.thumbnail_url.__func__.short_description = _("Thumbnail")
        self.dataset_link.__func__.short_description = _("Dataset")
        self.uploaded_files.__func__.short_description = _("Uploaded Files")
        self.upload_files.__func__.short_description = _("Upload Raster Files")
        self.preview_layer.__func__.short_description = _("Preview")
        self.mapviewer_map_url.__func__.short_description = _("MapViewer")

    def thumbnail_url(self, obj):
        raster_file = obj.raster_files.first()

        if raster_file:
            latest_upload_time = raster_file.time.strftime("%Y-%m-%d %H:%M:%S")

            return mark_safe(f"""
            <div class="thumbnail">
                <img src="{raster_file.thumbnail_url}" width="100" height="100">
                <div class="thumbnail__caption">
                    <b> Latest Upload: {latest_upload_time}</b> 
                </div>
            </div>
            """)

        return None

    def mapviewer_map_url(self, obj):
        label = _("MapViewer")
        if obj.raster_files_count == 0:
            return None

        button_html = f"""
            <a href="{obj.mapviewer_map_url}" target="_blank" rel="noopener noreferrer" class="button button-small button--icon button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-map icon" aria-hidden="true">
                        <use href="#icon-map"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)

    def dataset_link(self, obj):
        button_html = f"""
        <a href="{obj.dataset.dataset_url()}">
        {obj.dataset.title}
        </a>
        """
        return mark_safe(button_html)

    def upload_files(self, obj):
        label = _("Upload Files")
        button_html = f"""
            <a href="{obj.upload_url}" class="button button-small bicolor button--icon">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-upload"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)

    def uploaded_files(self, obj):
        label = _("View Uploaded Files")

        if obj.raster_files_count == 0:
            return None

        button_html = f"""
            <a href="{obj.get_uploads_list_url()}" class="button button-small button--icon bicolor button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-list-ol icon" aria-hidden="true">
                        <use href="#icon-list-ol"></use>
                    </svg>
                </span>
                {label} ({obj.raster_files_count})
            </a>
        """
        return mark_safe(button_html)

    def preview_layer(self, obj):
        label = _("Preview Layer")

        if obj.raster_files_count == 0:
            return None

        button_html = f"""
            <a href="{obj.preview_url}" class="button button-small button--icon button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-view"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)


class RasterFileIndexView(IndexView):
    def get_context_data(self, **kwargs):
        context_data = super(RasterFileIndexView, self).get_context_data(**kwargs)

        model_verbose_name = self.model._meta.verbose_name_plural

        category_admin_helper = AdminURLHelper(Category)
        categories_url = category_admin_helper.get_action_url("index")

        dataset_admin_helper = AdminURLHelper(Dataset)
        datasets_url = dataset_admin_helper.get_action_url("index")

        layer_admin_helper = AdminURLHelper(RasterFileLayer)
        layers_url = layer_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": categories_url, "label": _("Categories")},
            {"url": datasets_url, "label": _("Datasets")},
            {"url": layers_url, "label": _("Raster File Layers")},
            {"url": "#", "label": model_verbose_name},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class RasterFileEditView(EditView):
    def get_context_data(self, **kwargs):
        context_data = super(RasterFileEditView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        categories_url = category_admin_helper.get_action_url("index")

        dataset_admin_helper = AdminURLHelper(Dataset)
        datasets_url = dataset_admin_helper.get_action_url("index")

        layer_admin_helper = AdminURLHelper(RasterFileLayer)
        layers_url = layer_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": categories_url, "label": _("Categories")},
            {"url": datasets_url, "label": _("Datasets")},
            {"url": layers_url, "label": _("Raster File Layers")},
            {"url": "#", "label": self.instance}
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class RasterFileModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = LayerRasterFile
    exclude_from_explorer = True
    hidden = True

    list_display = ("thumbnail", "__str__", "layer", "time",)
    list_filter = ("layer",)
    list_display_add_buttons = "__str__"
    index_template_name = "geomanager/modeladmin/index_without_custom_create.html"

    index_view_class = RasterFileIndexView
    edit_view_class = RasterFileEditView
    delete_view_class = LayerFileDeleteView

    def thumbnail(self, obj):
        return mark_safe(f"""
            <a href="{obj.thumbnail_url}" target="_blank">
                <img src="{obj.thumbnail_url}" width="100" height="100" />
            </a>""")


urls = [
    path('upload-rasters/', upload_raster_file, name='geomanager_upload_rasters'),
    path('upload-rasters/<uuid:dataset_id>/', upload_raster_file, name='geomanager_dataset_upload_raster'),
    path('upload-rasters/<uuid:dataset_id>/<uuid:layer_id>/', upload_raster_file,
         name='geomanager_dataset_layer_upload_raster'),

    path('publish-rasters/<int:upload_id>/', publish_raster, name='geomanager_publish_raster'),
    path('delete-raster-upload/<int:upload_id>/', delete_raster_upload, name='geomanager_delete_raster_upload'),

    path('preview-raster-layers/<uuid:dataset_id>/', preview_raster_layers,
         name='geomanager_preview_raster_dataset'),
    path('preview-raster-layers/<uuid:dataset_id>/<uuid:layer_id>/', preview_raster_layers,
         name='geomanager_preview_raster_layer'),
]



from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from rest_framework.decorators import api_view, renderer_classes
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response

from geomanager.forms import GeoManagerUserProfileForm
from geomanager.models import GeoManagerUserProfile
from geomanager.serializers.profile import GeoManagerUserProfileSerializer


@api_view(['GET'])
@renderer_classes([JSONRenderer])
def get_geomanager_user_profile(request, user_id):
    try:
        profile = GeoManagerUserProfile.objects.get(user=user_id)
    except ObjectDoesNotExist:
        return Response({"detail": "Profile does not exist for user"}, status=404)

    serializer = GeoManagerUserProfileSerializer(profile)

    return Response(serializer.data)


@api_view(['PATCH'])
@renderer_classes([JSONRenderer])
def create_or_update_geomanager_user_profile(request, user_id):
    try:
        profile = GeoManagerUserProfile.objects.get(user=user_id)
    except ObjectDoesNotExist:
        profile = None

    try:
        user = get_user_model().objects.get(id=user_id)
    except ObjectDoesNotExist:
        return Response({"detail": "User does not exist"}, status=404)

    user_data_fields = ["first_name", "last_name"]

    data = request.data
    data.update({"user": user_id})

    if profile:
        form = GeoManagerUserProfileForm(request.data, instance=profile)
    else:
        form = GeoManagerUserProfileForm(request.data)

    if form.is_valid():
        profile = form.save()
        user_data = {}
        for key, value in request.data.items():
            if key in user_data_fields and value:
                user_data.update({key: value})
        if user_data:
            for key, value in user_data.items():
                setattr(user, key, value)
            user.save()

        profile.user = user
        serializer = GeoManagerUserProfileSerializer(profile)

        return Response(serializer.data)

    return Response({"detail": "Error occurred. Please try again later"}, status=400)



from django.apps import AppConfig
from django.db import connection
from django.db.utils import OperationalError

from .settings import geomanager_settings


class GeomanagerConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'geomanager'

    def ready(self):
        vector_db_schema = geomanager_settings.get("vector_db_schema")
        ensure_pg_service_schema_exists(vector_db_schema)


def ensure_pg_service_schema_exists(schema):
    if schema is not None:
        with connection.cursor() as cursor:
            try:
                schema_sql = f"""DO
                                    $do$
                                    BEGIN
                                        CREATE EXTENSION IF NOT EXISTS postgis;
                                        CREATE SCHEMA IF NOT EXISTS {schema};
                                    END
                                    $do$;"""
                cursor.execute(schema_sql)
            except OperationalError:
                # If the schema already exists, do nothing
                pass



from io import BytesIO
from xml.dom import minidom

from cairosvg import svg2png
from django.utils.safestring import mark_safe
from wagtailiconchooser.utils import get_svg_icons


def rasterize_svg_to_png(icon_name, fill_color=None):
    svg_icons = get_svg_icons()

    svg_str = svg_icons.get(icon_name)

    if not svg_str:
        return None

    doc = minidom.parseString(svg_str)
    svg = doc.getElementsByTagName("svg")
    if svg:
        svg = svg[0]

        svg.setAttribute("height", "26")
        svg.setAttribute("width", "26")

        if fill_color:
            svg.setAttribute("fill", fill_color)
            svg_str = mark_safe(svg.toxml())

    svg_bytes = svg_str.encode('utf-8')

    in_buf = BytesIO(svg_bytes)

    # Prepare a buffer for output
    out_buf = BytesIO()

    # Rasterise the SVG to PNG
    svg2png(file_obj=in_buf, write_to=out_buf)
    out_buf.seek(0)

    # Return a Willow PNGImageFile
    return out_buf



# Generated by Django 4.1.10 on 2023-11-10 11:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0030_delete_stationsettings'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='geomanagersettings',
            name='cap_auto_refresh_interval',
        ),
        migrations.RemoveField(
            model_name='geomanagersettings',
            name='cap_base_url',
        ),
        migrations.RemoveField(
            model_name='geomanagersettings',
            name='cap_metadata',
        ),
        migrations.RemoveField(
            model_name='geomanagersettings',
            name='cap_shown_by_default',
        ),
        migrations.RemoveField(
            model_name='geomanagersettings',
            name='cap_sub_category',
        ),
        migrations.RemoveField(
            model_name='geomanagersettings',
            name='gadm_version',
        ),
        migrations.AlterField(
            model_name='geomanagersettings',
            name='max_upload_size_mb',
            field=models.IntegerField(default=100, help_text='Maximum raster file size that can be uploaded in MegaBytes. Default is 100Mbs.', verbose_name='Maximum upload size in MegaBytes'),
        ),
    ]



import os

from django.conf import settings
from django.core.files.storage import FileSystemStorage


class OverwriteStorage(FileSystemStorage):
    def exists(self, name):
        exists = os.path.lexists(self.path(name))
        if exists:
            os.remove(os.path.join(settings.MEDIA_ROOT, name))
        return exists



from django.urls import path
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail_modeladmin.helpers import AdminURLHelper
from wagtail_modeladmin.views import CreateView, EditView

from geomanager.admin.base import BaseModelAdmin, ModelAdminCanHide, LayerIndexView
from geomanager.models import Dataset, WmsLayer, Category
from geomanager.views import preview_wms_layers


class WmsLayerCreateView(CreateView):
    def get_form(self):
        form = super().get_form()
        form.fields["dataset"].queryset = Dataset.objects.filter(layer_type="wms")

        dataset_id = self.request.GET.get("dataset_id")
        if dataset_id:
            initial = {**form.initial}
            initial.update({"dataset": dataset_id})
            form.initial = initial
        return form

    def get_context_data(self, **kwargs):
        context_data = super(WmsLayerCreateView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": "#", "label": _("New") + f" {WmsLayer._meta.verbose_name}"},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class WMSLayerEditView(EditView):
    def get_context_data(self, **kwargs):
        context_data = super(WMSLayerEditView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        layer_admin_helper = AdminURLHelper(WmsLayer)
        layer_index_url = layer_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": layer_index_url, "label": WmsLayer._meta.verbose_name_plural},
            {"url": "#", "label": self.instance.title},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class WmsLayerModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = WmsLayer

    exclude_from_explorer = True
    hidden = True

    index_template_name = "geomanager/modeladmin/index_without_custom_create.html"

    index_view_class = LayerIndexView
    create_view_class = WmsLayerCreateView
    edit_view_class = WMSLayerEditView

    def __init__(self, parent=None):
        super().__init__(parent)
        self.list_display = (list(self.list_display) or []) + ['dataset_link', 'preview_layer', "mapviewer_map_url"]
        self.dataset_link.__func__.short_description = _('Dataset')
        self.preview_layer.__func__.short_description = _('Preview on Map')
        self.mapviewer_map_url.__func__.short_description = _("View on MapViewer")

    def mapviewer_map_url(self, obj):
        label = _("MapViewer")
        button_html = f"""
            <a href="{obj.mapviewer_map_url}" target="_blank" rel="noopener noreferrer" class="button button-small button--icon button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-map icon" aria-hidden="true">
                        <use href="#icon-map"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)

    def dataset_link(self, obj):
        button_html = f"""
            <a href="{obj.dataset.dataset_url()}">
                {obj.dataset.title}
            </a>
        """
        return mark_safe(button_html)

    def preview_layer(self, obj):
        label = _("Preview Layer")
        button_html = f"""
            <a href="{obj.preview_url}" class="button button-small button--icon button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-view"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)


urls = [
    path('preview-wms-layers/<uuid:dataset_id>/', preview_wms_layers, name='geomanager_preview_wms_dataset'),
    path('preview-wms-layers/<uuid:dataset_id>/<uuid:layer_id>/', preview_wms_layers,
         name='geomanager_preview_wms_layer'),
]



from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from wagtail_modeladmin.helpers import AdminURLHelper
from wagtail_modeladmin.options import ModelAdmin
from wagtail_modeladmin.views import IndexView, DeleteView

from geomanager.models import Dataset, Category


class BaseModelAdmin(ModelAdmin):
    index_template_name = "geomanager/modeladmin/index.html"
    create_template_name = "geomanager/modeladmin/create.html"
    edit_template_name = "geomanager/modeladmin/edit.html"


class ModelAdminCanHide(ModelAdmin):
    hidden = False


class LayerIndexView(IndexView):
    def get_context_data(self, **kwargs):
        context_data = super(LayerIndexView, self).get_context_data(**kwargs)

        model_verbose_name = self.model._meta.verbose_name_plural

        dataset_admin_helper = AdminURLHelper(Dataset)
        datasets_url = dataset_admin_helper.get_action_url("index")

        category_admin_helper = AdminURLHelper(Category)
        categories_url = category_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": categories_url, "label": _("Categories")},
            {"url": datasets_url, "label": _("Datasets")},
            {"url": "#", "label": model_verbose_name},
        ]

        context_data.update({
            "custom_create_url": {
                "label": _("Create from datasets"),
                "url": datasets_url
            },
            "navigation_items": navigation_items,
        })

        return context_data


class LayerFileDeleteView(DeleteView):
    # update index url to add layer__id,
    # so that we redirect to raster files list filtered to
    # this instance's layer id, the url probably used to arrive to this list
    @cached_property
    def index_url(self):
        index_url = self.url_helper.index_url
        if self.instance:
            layer_id = str(self.instance.layer.pk)
            index_url += f"?layer__id={layer_id}"

        return index_url



from adminboundarymanager.models import AdminBoundary
from rest_framework import serializers

from geomanager.models.vector_file import (
    PgVectorTable,
    VectorFileLayer,
)


class AdminBoundarySerializer(serializers.ModelSerializer):
    bbox = serializers.ListField()

    class Meta:
        model = AdminBoundary
        fields = ("level", "name_0", "name_1", "name_2", "gid_0", "gid_1", "gid_2", "bbox")


class BoundsFieldSerializer(serializers.Field):
    def to_representation(self, value):
        # Convert the value of the custom field to a string for serialization
        return [float(value) for value in list(value)]


class PgVectorTableSerializer(serializers.ModelSerializer):
    bounds = BoundsFieldSerializer()

    class Meta:
        model = PgVectorTable
        fields = '__all__'


class VectorFileLayerSerializer(serializers.ModelSerializer):
    name = serializers.SerializerMethodField()
    layerConfig = serializers.SerializerMethodField()
    layerType = serializers.SerializerMethodField()
    params = serializers.SerializerMethodField()
    paramsSelectorConfig = serializers.SerializerMethodField()
    legendConfig = serializers.SerializerMethodField()
    multiTemporal = serializers.SerializerMethodField()
    currentTimeMethod = serializers.SerializerMethodField()
    autoUpdateInterval = serializers.SerializerMethodField()
    isMultiLayer = serializers.SerializerMethodField()
    nestedLegend = serializers.SerializerMethodField()
    interactionConfig = serializers.SerializerMethodField()
    canClip = serializers.SerializerMethodField()

    class Meta:
        model = VectorFileLayer
        fields = ["id", "dataset", "name", "layerType", "multiTemporal", "isMultiLayer", "legendConfig", "nestedLegend",
                  "layerConfig", "params", "paramsSelectorConfig", "currentTimeMethod", "autoUpdateInterval",
                  "interactionConfig", "canClip"]

    def get_isMultiLayer(self, obj):
        return obj.dataset.multi_layer

    def get_nestedLegend(self, obj):
        return obj.dataset.multi_layer

    def get_autoUpdateInterval(self, obj):
        return obj.dataset.auto_update_interval_milliseconds

    def get_multiTemporal(self, obj):
        return obj.dataset.multi_temporal

    def get_currentTimeMethod(self, obj):
        return obj.dataset.current_time_method

    def get_layerConfig(self, obj):
        request = self.context.get('request')

        layer_config = obj.layer_config(request)
        return layer_config

    def get_layerType(self, obj):
        return obj.dataset.layer_type

    def get_name(self, obj):
        return obj.title

    def get_params(self, obj):
        return obj.params

    def get_paramsSelectorConfig(self, obj):
        return obj.param_selector_config

    def get_paramsSelectorColumnView(self, obj):
        return not obj.params_selectors_side_by_side

    def get_legendConfig(self, obj):
        request = self.context.get('request')
        return obj.get_legend_config(request)

    def get_getCapabilitiesUrl(self, obj):
        return obj.get_capabilities_url

    def get_layerName(self, obj):
        return obj.layer_name

    def get_interactionConfig(self, obj):
        return obj.interaction_config

    def get_canClip(self, obj):
        return obj.dataset.can_clip



# Generated by Django 4.1.10 on 2023-11-01 10:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0027_alter_layerrasterfile_options_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='pgvectortable',
            options={'ordering': ['-time'], 'verbose_name': 'Vector Upload', 'verbose_name_plural': 'Vector Uploads'},
        ),
        migrations.AlterField(
            model_name='dataset',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='datasets', to='geomanager.category', verbose_name='category'),
        ),
    ]



from django.contrib.admin.utils import quote
from django.urls import reverse
from wagtail_modeladmin.helpers import AdminURLHelper


def get_layer_action_url(layer_type, action, action_args=None):
    if layer_type == "raster_file":
        from geomanager.models import RasterFileLayer
        file_layer_admin_helper = AdminURLHelper(RasterFileLayer)
        url = file_layer_admin_helper.get_action_url(action, action_args)
    elif layer_type == "vector_file":
        from geomanager.models.vector_file import VectorFileLayer
        vector_layer_admin_helper = AdminURLHelper(VectorFileLayer)
        url = vector_layer_admin_helper.get_action_url(action, action_args)
    elif layer_type == "wms":
        from geomanager.models.wms import WmsLayer
        wms_layer_admin_helper = AdminURLHelper(WmsLayer)
        url = wms_layer_admin_helper.get_action_url(action, action_args)
    elif layer_type == "raster_tile":
        from geomanager.models.raster_tile import RasterTileLayer
        raster_tile_layer_admin_helper = AdminURLHelper(RasterTileLayer)
        url = raster_tile_layer_admin_helper.get_action_url(action, action_args)
    elif layer_type == "vector_tile":
        from geomanager.models.vector_tile import VectorTileLayer
        vector_tile_layer_admin_helper = AdminURLHelper(VectorTileLayer)
        url = vector_tile_layer_admin_helper.get_action_url(action, action_args)
    else:
        url = None

    return url


def get_preview_url(layer_type, dataset_id, layer_id=None):
    args = [quote(dataset_id)]
    if layer_id:
        args.append(layer_id)

    if layer_type == "raster_file":
        if layer_id:
            preview_url = reverse(
                f"geomanager_preview_raster_layer",
                args=args,
            )
        else:
            preview_url = reverse(
                f"geomanager_preview_raster_dataset",
                args=args,
            )
    elif layer_type == "vector_file":
        if layer_id:
            preview_url = reverse(
                f"geomanager_preview_vector_layer",
                args=args,
            )
        else:
            preview_url = reverse(
                f"geomanager_preview_vector_dataset",
                args=args,
            )

    elif layer_type == "wms":
        if layer_id:
            preview_url = reverse(
                f"geomanager_preview_wms_layer",
                args=args,
            )
        else:
            preview_url = reverse(
                f"geomanager_preview_wms_dataset",
                args=args,
            )
    elif layer_type == "vector_tile":
        if layer_id:
            preview_url = reverse(
                f"geomanager_preview_vector_tile_layer",
                args=args,
            )
        else:
            preview_url = reverse(
                f"geomanager_preview_vector_tile_dataset",
                args=args,
            )
    elif layer_type == "raster_tile":
        if layer_id:
            preview_url = reverse(
                f"geomanager_preview_raster_tile_layer",
                args=args,
            )
        else:
            preview_url = reverse(
                f"geomanager_preview_raster_tile_dataset",
                args=args,
            )

    else:
        preview_url = None

    return preview_url


def get_upload_url(layer_type, dataset_id, layer_id=None):
    args = [quote(dataset_id)]
    if layer_id:
        args.append(layer_id)

    if layer_type == "raster_file":
        if layer_id:
            upload_url = reverse(
                f"geomanager_dataset_layer_upload_raster",
                args=args,
            )
        else:
            upload_url = reverse(
                f"geomanager_dataset_upload_raster",
                args=args,
            )
    elif layer_type == "vector_file":
        if layer_id:
            upload_url = reverse(
                f"geomanager_dataset_layer_upload_vector",
                args=args,
            )
        else:
            upload_url = reverse(
                f"geomanager_dataset_upload_vector",
                args=args,
            )
    else:
        upload_url = None

    return upload_url


def get_raster_layer_files_url(layer_id=None):
    from geomanager.models import LayerRasterFile
    admin_helper = AdminURLHelper(LayerRasterFile)
    url = admin_helper.get_action_url("index")

    if layer_id:
        url = url + f"?layer__id={layer_id}"

    return url


def get_vector_layer_files_url(layer_id=None):
    from geomanager.models import PgVectorTable
    admin_helper = AdminURLHelper(PgVectorTable)
    url = admin_helper.get_action_url("index")

    if layer_id:
        url = url + f"?layer__id={layer_id}"

    return url



from rest_framework import serializers

from geomanager.models.aoi import AreaOfInterest


class AoiSerializer(serializers.ModelSerializer):
    email = serializers.SerializerMethodField()

    class Meta:
        model = AreaOfInterest
        fields = ["id", "user", "email", "name", "geostore_id", "adm_0", "adm_1", "adm_2", "adm_3", "public", "type",
                  "tags", "webhook_url"]

    def to_representation(self, instance):
        aoi = super().to_representation(instance)
        if aoi.get("tags"):
            aoi["tags"] = aoi["tags"].split(",")
        return aoi

    def get_email(self, obj):
        return obj.user.email



# Generated by Django 4.1.10 on 2023-10-04 09:27

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks
import wagtail_color_panel.blocks
import wagtailiconchooser.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0023_vectortilelayericon'),
    ]

    operations = [
        migrations.AddField(
            model_name='rastertilelayer',
            name='get_time_from_tile_json',
            field=models.BooleanField(default=False, verbose_name='Get time from tile json url'),
        ),
        migrations.AddField(
            model_name='rastertilelayer',
            name='tile_json_url',
            field=models.URLField(blank=True, max_length=500, null=True, verbose_name='Tile JSON url'),
        ),
        migrations.AddField(
            model_name='rastertilelayer',
            name='timestamps_response_object_key',
            field=models.CharField(blank=True, default='timestamps', help_text='Key for timestamps values in response object', max_length=100, null=True, verbose_name='Timestamps response object key'),
        ),
        migrations.AddField(
            model_name='vectortilelayer',
            name='get_time_from_tile_json',
            field=models.BooleanField(default=False, verbose_name='Get time from tile json url'),
        ),
        migrations.AddField(
            model_name='vectortilelayer',
            name='tile_json_url',
            field=models.URLField(blank=True, max_length=500, null=True, verbose_name='Tile JSON url'),
        ),
        migrations.AddField(
            model_name='vectortilelayer',
            name='timestamps_response_object_key',
            field=models.CharField(blank=True, default='timestamps', help_text='Key for timestamps values in response object', max_length=100, null=True, verbose_name='Timestamps response object key'),
        ),
        migrations.AlterField(
            model_name='rastertilelayer',
            name='legend',
            field=wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Custom Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Custom Image')), ('legend_icon', wagtail.blocks.StructBlock([('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_label', wagtail.blocks.CharBlock(label='Icon Label')), ('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon color', required=False))]), label='Legend Icons', min_num=1))], label='Legend Icon'))], blank=True, null=True, use_json_field=True, verbose_name='Legend'),
        ),
        migrations.AlterField(
            model_name='vectortilelayer',
            name='legend',
            field=wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Custom Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Custom Image')), ('legend_icon', wagtail.blocks.StructBlock([('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('icon_image', wagtailiconchooser.blocks.IconChooserBlock(label='Icon Image')), ('icon_label', wagtail.blocks.CharBlock(label='Icon Label')), ('icon_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', label='Icon color', required=False))]), label='Legend Icons', min_num=1))], label='Legend Icon'))], blank=True, null=True, use_json_field=True, verbose_name='Legend'),
        ),
    ]



# Generated by Django 4.2.1 on 2023-06-09 10:15

from django.db import migrations, models
import geomanager.models.raster_file
import wagtail.blocks
import wagtail.fields
import wagtail_color_panel.fields


class Migration(migrations.Migration):
    dependencies = [
        ('geomanager', '0002_geomanagersettings_logo_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='colorvalue',
            name='label',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Label for Legend'),
        ),
        migrations.AlterField(
            model_name='colorvalue',
            name='threshold',
            field=models.FloatField(
                help_text='Values less than or equal to the input value, will be assigned the chosen color',
                verbose_name='Threshold value'),
        ),
        migrations.AlterField(
            model_name='geomanagersettings',
            name='navigation',
            field=wagtail.fields.StreamField([('menu_items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock(
                [('label', wagtail.blocks.CharBlock(label='Label')),
                 ('page', wagtail.blocks.PageChooserBlock(label='Page'))], max_num=8)))], blank=True, null=True,
                                             use_json_field=True),
        ),
        migrations.AlterField(
            model_name='layerrasterfile',
            name='file',
            field=models.FileField(upload_to=geomanager.models.raster_file.layer_raster_file_dir_path, verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='rasterstyle',
            name='custom_color_for_rest',
            field=wagtail_color_panel.fields.ColorField(blank=True, default='#ff0000',
                                                        help_text='Color for values greater than the values defined above, as well as values greater than the maximum defined value',
                                                        max_length=7, null=True,
                                                        verbose_name='Color for the rest of values'),
        ),
    ]



import logging
import os
import re
import tempfile
import uuid
from datetime import datetime
from os.path import splitext, isfile

import pytz
from adminboundarymanager.models import AdminBoundarySettings, AdminBoundary
from dateutil.parser import isoparse
from django.core.files import File
from django.db import transaction
from shapely import wkb
from wagtail.models import Site

from geomanager.models import RasterUpload, LayerRasterFile, RasterFileLayer
from geomanager.utils.raster_utils import (
    create_layer_raster_file,
    read_raster_info,
    bounds_to_polygon,
    check_raster_bounds_with_boundary,
    clip_netcdf,
    clip_geotiff
)

logger = logging.getLogger("geomanager.ingest")
logger.setLevel(logging.INFO)

ALLOWED_RASTER_FILE_EXTENSIONS = ['.tif', '.nc']


class IngestException(Exception):
    def __init__(self, message):
        self.message = message


def is_valid_uuid(val):
    try:
        uuid.UUID(str(val))
        return True
    except ValueError:
        return False


def extract_iso_date_from_filename(file_name):
    pattern = r"\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$"  # Pattern 'YYYY-MM-DDTHH:MM:SS.sssZ'
    match = re.search(pattern, file_name)  # Search for the pattern at the end of the file name

    if match:
        iso_date = match.group()  # Extract the matched ISO format string
        tz_unaware_date = isoparse(iso_date)  # Convert the string to datetime object
        tz_aware_date = tz_unaware_date.replace(tzinfo=pytz.UTC)  # Make the datetime object timezone aware
        return tz_aware_date
    else:
        return None  # Return None if the file name doesn't end with the specified format


def clip_raster_upload_to_boundary(upload, request=None):
    if request:
        abm_settings = AdminBoundarySettings.for_request(request)
    else:
        site = Site.objects.get(is_default_site=True)
        if not site:
            return None
        abm_settings = AdminBoundarySettings.for_site(site)

    abm_extents = abm_settings.combined_countries_bounds
    abm_countries = abm_settings.countries_list

    if not abm_extents:
        return upload

    raster_metadata = read_raster_info(upload.file.path)
    raster_bounds = raster_metadata.get("bounds")
    if not raster_bounds:
        return upload
    intersects_with_boundary, completely_inside_boundary = check_raster_bounds_with_boundary(raster_bounds, abm_extents)

    if not intersects_with_boundary:
        return upload

    if completely_inside_boundary:
        return upload

    raster_driver = raster_metadata.get("driver")
    country_geoms = []
    for country in abm_countries:
        code = country.get("code")
        alpha3 = country.get("alpha3")

        # query using code (2-letter code)
        country_boundary = AdminBoundary.objects.filter(level=0, gid_0=code).first()

        # query using alpha 3 (3-letter code)
        if not country_boundary:
            country_boundary = AdminBoundary.objects.filter(level=0, gid_0=alpha3).first()

        if country_boundary:
            shapely_geom = wkb.loads(country_boundary.geom.hex)
            country_geoms.append(shapely_geom)
        else:
            # use bbox instead
            bbox = country.get("bbox")
            if bbox:
                bounds_geom = bounds_to_polygon(bbox)
                country_geoms.append(bounds_geom)

    union_polygon = country_geoms[0]
    for polygon in country_geoms[1:]:
        union_polygon = union_polygon.union(polygon)

    if raster_driver == "netCDF":
        clip_fn = clip_netcdf
        suffix = ".nc"
    else:
        clip_fn = clip_geotiff
        suffix = ".tif"

    with tempfile.NamedTemporaryFile(suffix=suffix) as f:
        clipped_raster = clip_fn(upload.file.path, union_polygon, f.name)
        raster_metadata = read_raster_info(clipped_raster)

        with open(clipped_raster, 'rb') as clipped_file:
            file_obj = File(clipped_file, name=f.name)
            upload.file.save(upload.file.name, file_obj, save=True)

    upload.raster_metadata = raster_metadata
    upload.save()

    return upload


def create_raster(layer_obj, upload, time, overwrite=False, band_index=None, data_variable=None):
    # check if raster file with this time already exists
    exists = LayerRasterFile.objects.filter(layer=layer_obj, time=time).exists()

    # return if raster file already exists and overwrite is False
    if exists and not overwrite:
        logger.warning(f'LayerRasterFile for layer: {layer_obj.pk} and time: {time} already exists.')
        return

    # delete raster file if exists and overwrite is True, and create new raster file
    if exists and overwrite:
        with transaction.atomic():
            layer_raster_file = LayerRasterFile.objects.get(layer=layer_obj, time=time)
            layer_raster_file.delete()

            create_layer_raster_file(layer_obj, upload, time, band_index=band_index, data_variable=data_variable)
    else:
        # create new raster file
        create_layer_raster_file(layer_obj, upload, time, band_index=band_index, data_variable=data_variable)


def raw_raster_file_to_layer_raster_file(layer_obj, file_path, time=None, overwrite=False, clip_to_boundary=False):
    with open(file_path, "rb") as file:
        file_name = os.path.basename(file.name)

        upload = RasterUpload.objects.create(dataset=layer_obj.dataset)
        upload.file.save(file_name, file, save=True)

        raster_metadata = read_raster_info(upload.file.path)
        upload.raster_metadata = raster_metadata
        upload.save()

        try:
            if clip_to_boundary:
                # clip raster upload to boundary
                upload = clip_raster_upload_to_boundary(upload)

            raster_driver = raster_metadata.get("driver")

            if raster_driver == "netCDF":
                data_variable = layer_obj.auto_ingest_nc_data_variable

                if not data_variable:
                    raise IngestException(f'No NetCDF Auto ingestion data variable set for layer: {layer_obj}')

                if data_variable not in raster_metadata.get("data_variables", []):
                    raise IngestException(
                        f'NetCDF Auto ingestion data variable: {data_variable} not found in NetCDF: {file_name}')

                timestamps = raster_metadata.get("timestamps", None)

                if not timestamps:
                    raise IngestException(f'No timestamps found in NetCDF: {file_name}')

                for i, time_str in enumerate(timestamps):
                    d_time_unaware = datetime.fromisoformat(time_str)
                    d_time_aware = d_time_unaware.replace(tzinfo=pytz.UTC)

                    logger.info(f'[GEOMANAGER_AUTO_INGEST]: Processing time : {time_str}')

                    create_raster(layer_obj, upload, d_time_aware, overwrite=overwrite, band_index=i,
                                  data_variable=data_variable)

            elif raster_driver == "GTiff":
                if time:
                    logger.info(f'[GEOMANAGER_AUTO_INGEST]: Processing time : {time}')
                    create_raster(layer_obj, upload, time, overwrite=overwrite)

        finally:
            # delete raster upload
            upload.delete()


def ingest_raster_file(src_path, overwrite=False, clip_to_boundary=False):
    # Check if source path exists
    if not isfile(src_path):
        raise IngestException(f'File path: {src_path} does not exist.')

    file_extension = splitext(src_path)[1].lower()

    if file_extension not in ALLOWED_RASTER_FILE_EXTENSIONS:
        raise IngestException(f'File path: {src_path} is not a tiff or netcdf file.')

    directory = os.path.dirname(src_path)
    file_name = os.path.basename(src_path)
    file_name_without_extension = os.path.splitext(file_name)[0]

    # check if the directory name is an uuid
    layer_uuid = os.path.basename(os.path.normpath(directory))
    if not is_valid_uuid(layer_uuid):
        raise IngestException(f'Directory name: {directory} is not a valid uuid.')

    # check if layer exists
    raster_file_layer = RasterFileLayer.objects.filter(pk=layer_uuid).first()
    if not raster_file_layer:
        raise IngestException(f'RasterFileLayer with UUID: {layer_uuid} does not exist.')

    if file_extension == '.tif':
        # check if file name ends with iso format date, return the parsed date if it does
        iso_date_time = extract_iso_date_from_filename(file_name_without_extension)
        if not iso_date_time:
            raise IngestException(f'File name: {file_name} does not end with iso format date.')

        # create layer raster file from raw tiff file
        raw_raster_file_to_layer_raster_file(raster_file_layer, src_path, time=iso_date_time, overwrite=overwrite,
                                             clip_to_boundary=clip_to_boundary)

    elif file_extension == '.nc':
        # process netcdf file
        raw_raster_file_to_layer_raster_file(raster_file_layer, src_path, time=None, overwrite=overwrite,
                                             clip_to_boundary=clip_to_boundary)

    else:
        raise IngestException(f'File extension: {file_extension} not supported.')



# Generated by Django 4.1.10 on 2023-09-19 15:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0014_geomanagersettings_privacy_policy_page_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='wmslayer',
            name='request_time_from_capabilities',
            field=models.BooleanField(default=True, help_text='Get list of available times for this layer from getCapabilities', verbose_name='Request time from capabilities'),
        ),
    ]



import uuid

from django.conf import settings
from django.db import models
from django.utils.text import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel

from geomanager.models.geostore import Geostore


class AreaOfInterest(TimeStampedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, verbose_name=_("User"))

    name = models.CharField(max_length=255, verbose_name=_("Name"))
    geostore_id = models.ForeignKey(Geostore, blank=True, null=True, on_delete=models.SET_NULL,
                                    verbose_name=_("Geostore Id"))
    adm_0 = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("Adm 0"))
    adm_1 = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("Adm 1"))
    adm_2 = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("Adm 2"))
    adm_3 = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("Adm 3"))
    public = models.BooleanField(default=False, verbose_name=_("Public"))
    type = models.CharField(max_length=255, blank=True, null=True, verbose_name=_("Type"))
    products = models.TextField(blank=True, null=True, verbose_name=_("Products"))
    tags = models.TextField(blank=True, null=True, verbose_name=_("Tags"))
    webhook_url = models.URLField(blank=True, null=True, verbose_name=_("Webhook Url"))

    class Meta:
        verbose_name = _("Area of Interest")
        verbose_name_plural = _("Areas of Interest")

    def __str__(self):
        return self.name



# Generated by Django 4.1.10 on 2023-08-01 11:52

from django.db import migrations, models
import django.db.models.deletion
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0083_workflowcontenttype'),
        ('geomanager', '0012_geomanagersettings_crop_raster_to_country'),
    ]

    operations = [
        migrations.AddField(
            model_name='geomanagersettings',
            name='logo_external_link',
            field=models.URLField(blank=True, help_text='Used if internal page not provided', max_length=255, null=True, verbose_name='Logo external link'),
        ),
        migrations.AddField(
            model_name='geomanagersettings',
            name='logo_page',
            field=models.ForeignKey(blank=True, help_text='Internal page to navigate to on clicking the logo', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.page', verbose_name='Logo internal page link'),
        ),
        migrations.AlterField(
            model_name='geomanagersettings',
            name='navigation',
            field=wagtail.fields.StreamField([('menu_items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(label='Label')), ('page', wagtail.blocks.PageChooserBlock(help_text='Internal page to navigate', label='Page', required=False)), ('external_link', wagtail.blocks.URLBlock(label='External link to navigate to. Used if internal page not provided', required=False))], max_num=8)))], blank=True, null=True, use_json_field=True),
        ),
    ]



# Generated by Django 4.1.10 on 2023-11-15 10:07

from django.db import migrations, models
import geomanager.models.raster_file
import geomanager.storage


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0032_rasterfilelayer_auto_ingest_nc_data_variable'),
    ]

    operations = [
        migrations.AlterField(
            model_name='layerrasterfile',
            name='file',
            field=models.FileField(storage=geomanager.storage.OverwriteStorage, upload_to=geomanager.models.raster_file.layer_raster_file_dir_path, verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='rasterfilelayer',
            name='auto_ingest_nc_data_variable',
            field=models.CharField(blank=True, help_text='The name of the data variable to use, if ingesting from netCDF files', max_length=100, null=True, verbose_name='Data variable for netCDF data auto ingest'),
        ),
    ]



def create_boundary_dataset(tiles_url):
    dataset_id = "political-boundaries"
    name = "Political Boundaries"
    source_layer = "default"

    dataset = {
        "id": dataset_id,
        "dataset": dataset_id,
        "name": name,
        "layer": dataset_id,
        "isBoundary": True,
        "public": True,
        "layers": []
    }

    layer = {
        "id": dataset_id,
        "isBoundary": True,
        "analysisEndpoint": "admin",
        "name": name,
        "default": True,
        "layerConfig": {
            "type": "vector",
            "source": {
                "type": "vector",
                "tiles": [tiles_url],
            },
            "render": {
                "layers": [
                    {
                        "filter": ["==", "level", 0],
                        "maxzoom": 3,
                        "paint": {
                            "fill-color": "#ffffff",
                            "fill-opacity": 0,
                        },
                        "source-layer": source_layer,
                        "type": "fill",
                    },
                    {
                        "filter": ["==", "level", 1],
                        "maxzoom": 6,
                        "minzoom": 3,
                        "paint": {
                            "fill-color": "#ffffff",
                            "fill-opacity": 0,
                        },
                        "source-layer": source_layer,
                        "type": "fill",
                    },
                    {
                        "filter": ["==", "level", 2],
                        "minzoom": 6,
                        "paint": {
                            "fill-color": "#ffffff",
                            "fill-opacity": 0,
                        },
                        "source-layer": source_layer,
                        "type": "fill",
                    },
                    {
                        "filter": ["==", "level", 0],
                        "paint": {
                            "line-color": "#C0FF24",
                            "line-width": 1,
                            "line-offset": 1,
                        },
                        "source-layer": source_layer,
                        "type": "line",
                    },
                    {
                        "filter": ["==", "level", 0],
                        "paint": {
                            "line-color": "#000",
                            "line-width": 1.5,
                        },
                        "source-layer": source_layer,
                        "type": "line",
                    },
                    {
                        "filter": ["==", "level", 1],
                        "maxzoom": 6,
                        "minzoom": 3,
                        "paint": {
                            "line-color": "#8b8b8b",
                            "line-width": 1,
                        },
                        "source-layer": source_layer,
                        "type": "line",
                    },
                    {
                        "filter": ["==", "level", 2],
                        "minzoom": 6,
                        "paint": {
                            "line-color": "#444444",
                            "line-dasharray": [2, 4],
                            "line-width": 0.7,
                        },
                        "source-layer": source_layer,
                        "type": "line",
                    },
                ],
            },
        },
        "interactionConfig": {
            "output": [
                {
                    "column": "gid_0",
                    "property": "ISO",
                    'type': "string",
                },
                {
                    "column": "gid_1",

                    "property": "admin1",
                    "type": "string",
                },
                {
                    "column": "gid_2",
                    "property": "admin2",
                    "type": "string",
                },
                {
                    "column": "name_0",
                    "property": "Country",
                    "type": "string",
                },
                {
                    "column": "name_1",
                    "property": "Region",
                    "type": "string",
                },
                {
                    "column": "name_2",
                    "property": "Sub Region",
                    "type": "string",
                },
                {
                    "column": "level",
                    "property": "Admin Level",
                    "type": "number",
                },
                {
                    "column": "area",
                    "format": "0a",
                    "property": "Area",
                    "suffix": "ha",
                    "type": "number",
                },
                {
                    "column": "gid",
                    "hidden": True,
                    "property": "gid",
                    "type": "number",
                },
            ],
            "type": "intersection",
        },
    }

    dataset["layers"].append(layer)

    return dataset



from django import forms
from django.forms import ModelForm
from django.utils.translation import gettext_lazy as _
from wagtail.admin.forms import WagtailAdminModelForm

from geomanager.models.aoi import AreaOfInterest
from geomanager.models.profile import GeoManagerUserProfile


class LayerRasterFileForm(forms.Form):
    layer = forms.ModelChoiceField(required=True, queryset=None, empty_label=None, label=_("layer"), )
    time = forms.DateTimeField(required=True,
                               widget=forms.DateTimeInput(attrs={'type': 'datetime-local'}, ),
                               localize=False, label=_("time"),
                               help_text=_("Time for the raster file. This can be the time the data was acquired, "
                                           "or the date and time for which the data applies"))
    nc_dates = forms.MultipleChoiceField(required=False, widget=forms.CheckboxSelectMultiple, label=_("NetCDF Dates"),
                                         help_text=_("Timestamps in the dataset"))
    nc_data_variable = forms.CharField(required=False, widget=forms.HiddenInput())

    def __init__(self, nc_dates_choices=None, *args, **kwargs):

        queryset = kwargs.pop('queryset', None)
        super().__init__(*args, **kwargs)
        self.fields['layer'].queryset = queryset

        if nc_dates_choices:
            self.fields['nc_dates'].choices = [(choice, choice) for choice in nc_dates_choices]

            # hide time input
            self.fields['time'].widget = forms.HiddenInput()
            self.fields['time'].required = False
        else:
            self.fields['nc_dates'].widget = forms.HiddenInput()


class VectorLayerFileForm(forms.Form):
    layer = forms.ModelChoiceField(required=True, queryset=None, empty_label=None, label=_("layer"))
    time = forms.DateTimeField(required=True,
                               widget=forms.DateTimeInput(attrs={'type': 'datetime-local'}, ),
                               localize=False,
                               label=_("time"),
                               help_text=_("Time is required for every uploaded vector file. "
                                           "This can be the time the dataset was collected, or applies to."))
    table_name = forms.CharField(required=True, label=_("database table name"))
    description = forms.CharField(required=False, widget=forms.Textarea, label=_("dataset description"),
                                  help_text=_("optional dataset description"))

    def __init__(self, *args, **kwargs):
        queryset = kwargs.pop('queryset', None)
        super().__init__(*args, **kwargs)
        self.fields['layer'].queryset = queryset


class BoundaryUploadForm(forms.Form):
    remove_existing = forms.BooleanField(required=False, widget=forms.HiddenInput)
    geopackage = forms.FileField(required=True, label=_("GADM Country Geopackage"),
                                 help_text=_("The uploaded file should be a geopackage, "
                                             "downloaded from https://gadm.org/download_country.html"),
                                 widget=forms.FileInput(attrs={'accept': '.gpkg'}))


class RasterStyleModelForm(WagtailAdminModelForm):
    def is_valid(self):
        valid = super().is_valid()
        if not valid:
            return False

        use_custom_colors = self.cleaned_data.get("use_custom_colors")
        min_value = self.cleaned_data.get("min")
        max_value = self.cleaned_data.get("max")
        steps = self.cleaned_data.get("steps")
        custom_color_for_rest = self.cleaned_data.get("custom_color_for_rest")

        if max_value <= min_value:
            self.add_error("max", _("Maximum value should be greater than minimum value"))
            return False

        if min_value >= max_value:
            self.add_error("min", _("Minimum value should be less than maximum value"))
            return False

        if not use_custom_colors and not steps:
            self.add_error("steps", _("Steps required when not using custom colors"))
            return False

        if use_custom_colors:

            # no color for values greater than max
            if not custom_color_for_rest:
                self.add_error("custom_color_for_rest",
                               _("Color for the rest of values must be specified"))
                return False

            color_values_formset = self.formsets.get("color_values")

            initial_form_count = color_values_formset.initial_form_count()
            total_form_count = color_values_formset.total_form_count()
            deleted_forms_count = len(color_values_formset.deleted_forms)

            empty_forms_count = 0
            for i, form in enumerate(color_values_formset.forms):
                # Empty forms are unchanged forms beyond those with initial data.
                if not form.has_changed() and i >= initial_form_count:
                    empty_forms_count += 1

            # No custom value added
            if total_form_count - deleted_forms_count - empty_forms_count < 1:
                color_values_formset._non_form_errors = [
                    _("You selected to use custom colors but did not add any. Please add color values and save.")
                ]
                return False

            for form in color_values_formset:
                threshold = form.cleaned_data.get("threshold")

                if threshold and min_value and max_value:
                    if threshold < min_value:
                        form.add_error("threshold", _("Value must be greater than minimum defined value"))
                        return False
                    if threshold > max_value:
                        form.add_error("threshold", _("Value must be less than or equal to the maximum defined value"))
                        return False
        return True


class GeoManagerUserProfileForm(ModelForm):
    class Meta:
        model = GeoManagerUserProfile
        fields = "__all__"


class AoiForm(ModelForm):
    class Meta:
        model = AreaOfInterest
        fields = "__all__"


class VectorTableForm(forms.Form):
    columns = forms.JSONField(required=False, widget=forms.HiddenInput)



from rest_framework import serializers

from geomanager.models.geostore import Geostore
from geomanager.utils.vector_utils import create_feature_collection_from_geom


class GeostoreSerializer(serializers.ModelSerializer):
    attributes = serializers.SerializerMethodField()

    class Meta:
        model = Geostore
        fields = ["id", "attributes"]

    def get_attributes(self, obj):
        geostore = {
            'info': obj.info,
            'geojson': create_feature_collection_from_geom(obj.geom),
            'bbox': obj.bbox,
            'hash': obj.pk
        }

        return geostore



from django.conf import settings

geomanager_settings = {
    "vector_db_schema": getattr(settings, "GEOMANAGER_VECTOR_DB_SCHEMA", "vectordata"),
    "auto_ingest_raster_data_dir": getattr(settings, "GEOMANAGER_AUTO_INGEST_RASTER_DATA_DIR", None)
}



import logging

from django.core.management.base import BaseCommand

from geomanager.utils.ingest import ingest_raster_file

logger = logging.getLogger(__name__)

ALLOWED_FILE_EVENTS = ["created", "moved"]


class Command(BaseCommand):
    help = 'Ingest single geotiff file to raster file'

    def add_arguments(self, parser):
        parser.add_argument('event_type', type=str, help='WatchDog File Event type')
        parser.add_argument('src', type=str, help='File source path')
        parser.add_argument('--dst', type=str, help='File destination path for moved events')
        parser.add_argument('--overwrite', action='store_true', default=False, help='Overwrite existing raster file')
        parser.add_argument('--clip', action='store_true', default=False, help='Clip raster to county boundary')

    def handle(self, *args, **options):
        event_type = options['event_type']
        src_path = options['src']
        dst_path = options['dst']
        overwrite = options['overwrite']
        clip = options['clip']

        logger.info('[GEOMANAGER_AUTO_INGEST]: Starting auto ingest execution...')

        logger.info(f'[GEOMANAGER_AUTO_INGEST]: Event Type: {event_type}')

        # Check if event type is allowed
        if event_type not in ALLOWED_FILE_EVENTS:
            logger.warning(f'[GEOMANAGER_AUTO_INGEST]: Event Type: {event_type} not in allowed file events.')
            return

        # If event type is moved, use destination path
        if event_type == "moved" and dst_path is not None:
            src_path = dst_path

        ingest_raster_file(src_path, overwrite, clip)

        logger.info(f'[GEOMANAGER_AUTO_INGEST]: {src_path} done...')



from django_nextjs.render import render_nextjs_page_sync
from wagtail.api.v2.utils import get_full_url
from wagtailiconchooser.utils import get_svg_sprite_for_icons

from geomanager.models import GeomanagerSettings, Category
from geomanager.models.vector_file import get_legend_icons


def map_view(request, location_type=None, adm0=None, adm1=None, adm2=None):
    # get svg sprite for categories and legend icons
    category_icons = [category.icon for category in Category.objects.all()]
    legend_icons = get_legend_icons()
    icons = [*category_icons, *legend_icons]
    svg_sprite = get_svg_sprite_for_icons(icons)

    gm_settings = GeomanagerSettings.for_request(request)
    context = {
        "svg_sprite": svg_sprite,
        "logo": gm_settings.logo,
        "logo_url": get_full_url(request, "")
    }

    if gm_settings.logo_page:
        context.update({"logo_url": get_full_url(request, gm_settings.logo_page.url)})

    if not gm_settings.logo_page and gm_settings.logo_external_link:
        context.update({"logo_url": gm_settings.logo_external_link, "logo_url_external": True})

    for block in gm_settings.navigation:
        if block.block_type == "menu_items":
            context.update({"menu_items": block.value})

    return render_nextjs_page_sync(request, template_name="django_nextjs/mapviewer.html", context=context,
                                   allow_redirects=True)



import json
from uuid import UUID

from django.utils.translation import gettext_lazy as _


class UUIDEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, UUID):
            # if the obj is uuid, we simply return the value of uuid
            return str(obj)
        return json.JSONEncoder.default(self, obj)


DATE_FORMAT_CHOICES = (
    ("yyyy-MM-dd HH:mm", _("Hour minute:second - (E.g 2023-01-01 00:00)")),
    ("yyyy-MM-dd", _("Day - (E.g 2023-01-01)")),
    ("yyyy-MM", _("Month number - (E.g 2023-01)")),
    ("MMMM yyyy", _("Month name - (E.g January 2023)")),
    ("pentadal", _("Pentadal - (E.g Jan 2023 - P1 1-5th)"))
)



# Generated by Django 4.1.10 on 2023-09-19 16:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0015_wmslayer_request_time_from_capabilities'),
    ]

    operations = [
        migrations.AddField(
            model_name='wmslayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-mm-dd HH:MM', 'Hourly (2023-01-01 00:00)'), ('yyyy-mm-dd', 'Year-Month-Day (2023-01-01)'), ('yyyy-mm', 'Year-Month (2023-01-01)'), ('mm yyyy', 'MonthName Year (January 2023)'), ('pentadal', 'MonthName Year - Pentad number Dates  (Jan 2023 - P1 1-5th)')], default='yyyy-mm-dd HH:MM', max_length=100, null=True),
        ),
    ]



from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail_adminsortable.admin import SortableAdminMixin
from wagtail_modeladmin.helpers import AdminURLHelper, ButtonHelper
from wagtail_modeladmin.views import CreateView, EditView

from geomanager.admin.base import BaseModelAdmin, ModelAdminCanHide
from geomanager.models import Category


class CategoryCreateView(CreateView):
    def get_context_data(self, **kwargs):
        context_data = super(CategoryCreateView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": "#", "label": _("New") + f" {Category._meta.verbose_name}"},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class CategoryEditView(EditView):
    def get_context_data(self, **kwargs):
        context_data = super(CategoryEditView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": "#", "label": self.instance.title},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class CategoryButtonHelper(ButtonHelper):
    def get_buttons_for_obj(
            self, obj, exclude=None, classnames_add=None, classnames_exclude=None
    ):
        buttons = super().get_buttons_for_obj(obj, exclude, classnames_add, classnames_exclude)

        classnames = self.edit_button_classnames + classnames_add
        cn = self.finalise_classname(classnames, classnames_exclude)

        create_dataset_button = {
            "url": obj.dataset_create_url(),
            "label": _("Add Dataset"),
            "classname": cn,
            "title": _("Add Dataset") % {"object": self.verbose_name},
        }

        buttons.append(create_dataset_button)

        return buttons


class CategoryModelAdmin(SortableAdminMixin, BaseModelAdmin, ModelAdminCanHide):
    model = Category
    menu_label = _("Datasets")
    exclude_from_explorer = True
    button_helper_class = CategoryButtonHelper
    menu_icon = "layer-group"
    list_display_add_buttons = "__str__"

    create_view_class = CategoryCreateView
    edit_view_class = CategoryEditView

    def __init__(self, parent=None):
        super().__init__(parent)
        self.list_display = ["category_icon"] + (list(self.list_display) or []) + ["view_datasets", "create_dataset",
                                                                                   "mapviewer_map_url"]
        self.category_icon.__func__.short_description = _('Icon')
        self.create_dataset.__func__.short_description = _('Add Dataset')
        self.view_datasets.__func__.short_description = _('View Datasets')
        self.mapviewer_map_url.__func__.short_description = _("View on MapViewer")

    def category_icon(self, obj):
        icon = obj.icon
        if not obj.icon:
            icon = "layer-group"

        icon_html = f"""
           <span class="icon-wrapper">
                <svg class="icon icon-{icon} icon" aria-hidden="true" style="height:40px;width:40px">
                    <use href="#icon-{icon}"></use>
                </svg>
            </span>
        """
        return mark_safe(icon_html)

    def mapviewer_map_url(self, obj):
        label = _("MapViewer")
        button_html = f"""
                <a href="{obj.mapviewer_map_url}" target="_blank" rel="noopener noreferrer" class="button button-small button--icon button-secondary">
                    <span class="icon-wrapper">
                        <svg class="icon icon-map icon" aria-hidden="true">
                            <use href="#icon-map"></use>
                        </svg>
                    </span>
                    {label}
                </a>
            """
        return mark_safe(button_html)

    def create_dataset(self, obj):
        label = _("Add Dataset")
        button_html = f"""
            <a href="{obj.dataset_create_url()}" class="button button-small button--icon bicolor">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-plus"></use>
                    </svg>
                </span>
              {label}
            </a>
        """
        return mark_safe(button_html)

    def view_datasets(self, obj):
        label = _("View Datasets")

        datasets_count = obj.datasets.count()

        button_html = f"""
            <a href="{obj.datasets_list_url()}" class="button button-small button--icon bicolor button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-database icon" aria-hidden="true">
                        <use href="#icon-database"></use>
                    </svg>
                </span>
                {label} ({datasets_count})
            </a>
        """
        return mark_safe(button_html)



import os

from django.core.exceptions import ValidationError
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel, FieldRowPanel, MultiFieldPanel, InlinePanel
from wagtail.api.v2.utils import get_full_url
from wagtail.fields import StreamField
from wagtail.models import Orderable
from wagtail_color_panel.edit_handlers import NativeColorPanel
from wagtail_color_panel.fields import ColorField
from wagtail_modeladmin.helpers import AdminURLHelper

from geomanager.blocks import (
    FileLayerPointAnalysisBlock,
    FileLayerAreaAnalysisBlock
)
from geomanager.forms import RasterStyleModelForm
from geomanager.helpers import get_raster_layer_files_url
from geomanager.models.core import Dataset, BaseLayer
from geomanager.settings import geomanager_settings
from geomanager.storage import OverwriteStorage
from geomanager.utils import DATE_FORMAT_CHOICES
from geomanager.widgets import RasterStyleWidget


class RasterFileLayer(TimeStampedModel, BaseLayer):
    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name="raster_file_layers",
                                verbose_name=_("dataset"))
    date_format = models.CharField(max_length=100, choices=DATE_FORMAT_CHOICES, blank=True, null=True,
                                   default="yyyy-MM-dd HH:mm",
                                   verbose_name=_("Display Format for DateTime Selector"))
    style = models.ForeignKey("RasterStyle", null=True, blank=True, on_delete=models.SET_NULL, verbose_name=_("style"))

    auto_ingest_from_directory = models.BooleanField(default=False, verbose_name=_("Auto ingest from directory"))
    auto_ingest_nc_data_variable = models.CharField(max_length=100, blank=True, null=True,
                                                    verbose_name=_("Data variable for netCDF data auto ingest"),
                                                    help_text=_("The name of the data variable to use, "
                                                                "if ingesting from netCDF files"))

    analysis = StreamField([
        ('point_analysis', FileLayerPointAnalysisBlock(label=_("Point Analysis")),),
        ('area_analysis', FileLayerAreaAnalysisBlock(label=_("Area Analysis")),),
    ], block_counts={'point_analysis': {'max_num': 1}, 'area_analysis': {'max_num': 1}}, use_json_field=True,
        null=True, blank=True, max_num=2, verbose_name=_("Analysis"), )

    class Meta:
        verbose_name = _("Raster File Layer")
        verbose_name_plural = _("Raster File Layers")

    panels = [
        FieldPanel("dataset"),
        FieldPanel("title"),
        FieldPanel("default"),
        FieldPanel("date_format"),
        FieldPanel("style"),
        FieldPanel("auto_ingest_from_directory"),
        FieldPanel("auto_ingest_nc_data_variable"),
        FieldPanel("analysis"),
    ]

    def __str__(self):
        return f"{self.dataset.title} - {self.title}"

    @property
    def raster_files_count(self):
        return self.raster_files.count()

    def get_uploads_list_url(self):
        url = get_raster_layer_files_url(self.pk)
        return url

    def get_style_url(self):
        url = {"action": _("Create Style")}
        style_admin_helper = AdminURLHelper(RasterStyle)
        if self.style:
            url.update({
                "action": _("Edit Style"),
                "url": style_admin_helper.get_action_url("edit", self.style.pk)
            })
        else:
            url.update({
                "url": style_admin_helper.get_action_url("create") + f"?layer_id={self.pk}"
            })
        return url

    @property
    def base_tile_url(self):
        base_tiles_url = reverse("raster_tiles", args=(self.id, 0, 0, 0))
        base_tiles_url = base_tiles_url.replace("/0/0/0", r"/{z}/{x}/{y}")
        return f"{base_tiles_url}"

    def get_tile_json_url(self, request=None):
        tile_json_url = reverse("raster_file_tile_json", args=(self.id,))
        if request:
            tile_json_url = get_full_url(request, tile_json_url)

        return tile_json_url

    def layer_config(self, request=None):
        base_tile_url = self.base_tile_url
        if request:
            base_tile_url = get_full_url(request, base_tile_url)

        tile_url = f"{base_tile_url}?time={{time}}"

        if self.dataset.can_clip:
            tile_url = tile_url + "&geostore_id={geostore_id}"

        layer_config = {
            "type": "raster",
            "source": {
                "type": "raster",
                "tiles": [tile_url]
            }
        }

        return layer_config

    @property
    def params(self):
        params = {
            "time": ""
        }

        if self.dataset.can_clip:
            params.update({"geostore_id": ""})

        return params

    @property
    def param_selector_config(self):
        time_config = {
            "key": "time",
            "required": True,
            "sentence": "{selector}",
            "type": "datetime",
            "availableDates": [],
        }

        if self.date_format:
            if self.date_format == "pentadal":
                time_config.update({
                    "dateFormat": {"currentTime": "MMM yyyy", "asPeriod": "pentadal"},
                })
            else:
                time_config.update({
                    "dateFormat": {"currentTime": self.date_format},
                })
        else:
            time_config.update({
                "dateFormat": {"currentTime": "yyyy-MM-dd HH:mm"},
            })

        return [time_config]

    def get_legend_config(self):
        if self.style:
            return self.style.get_legend_config()

        config = {}
        return config

    def get_analysis_config(self):
        analysis_config = {}

        for analysis in self.analysis:
            data = analysis.block.get_api_representation(analysis.value)

            if analysis.block_type == "point_analysis":
                unit = data.get("unit")
                if data.get("instance_data_enabled"):
                    analysis_config.update({
                        "pointInstanceAnalysis": {"unit": unit}
                    })
                if data.get("timeseries_data_enabled"):
                    analysis_config.update({
                        "pointTimeseriesAnalysis": {
                            "unit": unit,
                            "chartType": data.get("timeseries_chart_type"),
                            "chartColor": data.get("timeseries_chart_color")
                        }
                    })
            if analysis.block_type == "area_analysis":
                unit = data.get("unit")
                if data.get("instance_data_enabled"):
                    analysis_config.update({
                        "areaInstanceAnalysis": {
                            "unit": unit,
                            "valueType": data.get("instance_value_type")
                        }
                    })
                if data.get("timeseries_data_enabled"):
                    analysis_config.update({
                        "areaTimeseriesAnalysis": {
                            "unit": unit,
                            "aggregationMethod": data.get("timeseries_aggregation_method"),
                            "chartType": data.get("timeseries_chart_type"),
                            "chartColor": data.get("timeseries_chart_color")
                        }
                    })

        return analysis_config

    def get_tile_json(self, request=None):
        base_tile_url = self.base_tile_url
        timestamps = list(self.raster_files.all().values_list("time", flat=True))

        if request:
            base_tile_url = get_full_url(request, base_tile_url)

        tile_json = {
            "tilejson": "3.0.0",
            "name": self.title,
            "scheme": "xyz",
            "tiles": [base_tile_url],
            "minzoom": 0,
            "maxzoom": 20,
            "time_parameter": "time",
            "timestamps": [t.strftime("%Y-%m-%dT%H:%M:%S.000Z") for t in timestamps]
        }

        return tile_json

    def clean(self):
        # if adding a layer to a dataset that already has a layer and is not multi layer
        if self._state.adding:
            if self.dataset.has_layers() and not self.dataset.multi_layer:
                raise ValidationError(_("Can not add layer because the dataset is not marked as Multi Layer. "
                                        "To add multiple layers to a dataset, please mark the dataset as "
                                        "Multi Layer and try again"))


def layer_raster_file_dir_path(instance, filename):
    file_dir = f"raster_files/{type(instance.layer).__name__}-{instance.layer.pk}/{filename}"
    return file_dir


class LayerRasterFile(TimeStampedModel):
    layer = models.ForeignKey(RasterFileLayer, on_delete=models.CASCADE, related_name="raster_files",
                              verbose_name=_("layer"))
    file = models.FileField(upload_to=layer_raster_file_dir_path, storage=OverwriteStorage, verbose_name=_("file"))
    time = models.DateTimeField(verbose_name=_("time"),
                                help_text=_("Time for the raster file. This can be the time the data was acquired, "
                                            "or the date and time for which the data applies", ))
    raster_metadata = models.JSONField(blank=True, null=True)

    class Meta:
        verbose_name = _("Layer Raster File")
        verbose_name_plural = _("Layer Raster Files")
        ordering = ["-time"]
        unique_together = ('layer', 'time')

    panels = [
        FieldPanel("time"),
        FieldPanel("raster_metadata", read_only=True),
    ]

    def __str__(self):
        return f"{self.layer.title} - {self.time}"

    @property
    def thumbnail_url(self):
        url = reverse("raster_file_thumbnail", kwargs={"file_id": self.pk})
        return url

    @property
    def time_str(self):
        return self.time.strftime("%Y-%m-%dT%H:%M:%S.000Z")


class RasterUpload(TimeStampedModel):
    dataset = models.ForeignKey(Dataset, blank=True, null=True, on_delete=models.SET_NULL, verbose_name=_("dataset"))
    file = models.FileField(upload_to="raster_uploads", verbose_name=_("file"))
    raster_metadata = models.JSONField(blank=True, null=True)

    class Meta:
        verbose_name = _("Raster Upload")
        verbose_name_plural = _("Raster Uploads")

    panels = [
        FieldPanel("layer"),
        FieldPanel("file"),
    ]

    def __str__(self):
        return f"{self.dataset} - {self.created}"


class RasterStyle(TimeStampedModel, ClusterableModel):
    base_form_class = RasterStyleModelForm

    name = models.CharField(max_length=256, verbose_name=_("name"),
                            help_text=_("Style name for identification"))
    unit = models.CharField(max_length=100, blank=True, null=True, verbose_name=_("data unit"),
                            help_text=_("Data unit"))
    min = models.IntegerField(default=0, verbose_name=_("minimum value"), help_text=_("minimum value"))
    max = models.IntegerField(default=100, verbose_name=_("maximum value"), help_text=_("maximum value"))
    steps = models.IntegerField(default=5, validators=[MinValueValidator(3), MaxValueValidator(20), ], null=True,
                                blank=True, verbose_name=_("steps"), help_text=_("Number of steps"))
    use_custom_colors = models.BooleanField(default=False, verbose_name=_("Use Custom Colors"))
    palette = models.TextField(blank=True, null=True, verbose_name=_("Color Palette"))
    interpolate = models.BooleanField(default=False, verbose_name=_("interpolate"), help_text="Interpolate colorscale")
    custom_color_for_rest = ColorField(blank=True, null=True, default="#ff0000",
                                       verbose_name=_("Color for the rest of values"),
                                       help_text=_(
                                           "Color for values greater than the values defined above, "
                                           "as well as values greater than the maximum defined value"))

    class Meta:
        verbose_name = _("Raster Style")
        verbose_name_plural = _("Raster Styles")

    def __str__(self):
        return self.name

    panels = [
        FieldPanel("name"),
        FieldPanel("unit"),
        FieldRowPanel(
            [
                FieldPanel("min"),
                FieldPanel("max"),
            ], _("Data values")
        ),
        FieldPanel("steps"),
        FieldPanel("palette", widget=RasterStyleWidget),
        FieldPanel("use_custom_colors"),
        MultiFieldPanel([
            InlinePanel("color_values", heading=_("Color Values"), label=_("Color Value")),
            NativeColorPanel("custom_color_for_rest"),
        ], _("Custom Color Values")),

        # FieldPanel("interpolate")
    ]

    def get_palette_list(self):
        if not self.use_custom_colors:
            return self.palette.split(",")
        return self.get_custom_palette()

    @property
    def min_value(self):
        return self.min

    @property
    def max_value(self):
        max_value = self.max
        if self.min == max_value:
            max_value += 0.1
        return max_value

    @property
    def scale_value(self):
        return 254 / (self.max_value - self.min_value)

    @property
    def offset_value(self):
        return -self.min_value

    @property
    def clip_value(self):
        return self.max_value + self.offset_value

    def get_custom_color_values(self):
        values = []
        color_values = self.color_values.order_by('threshold')

        for i, c_value in enumerate(color_values):
            value = c_value.value
            # if not the first one, add prev value for later comparison
            if i == 0:
                value["min_value"] = None
            else:
                value["min_value"] = color_values[i - 1].threshold
            value["max_value"] = value["threshold"]
            values.append(value)
        return values

    def get_custom_palette(self):
        colors = []
        for i in range(256):
            color = self.get_color_for_index(i)
            colors.append(color)

        return colors

    def get_color_for_index(self, index_value):
        values = self.get_custom_color_values()

        for value in values:
            max_value = value["max_value"] + self.offset_value

            if max_value > self.clip_value:
                max_value = self.clip_value

            if max_value < 0:
                max_value = 0

            max_value = self.scale_value * max_value

            if value["min_value"] is None:
                if index_value <= max_value:
                    return values[0]["color"]

            if value["min_value"] is not None:
                min_value = value["min_value"] + self.offset_value

                if min_value > self.clip_value:
                    min_value = self.clip_value

                if min_value < 0:
                    min_value = 0

                min_value = self.scale_value * min_value

                if min_value < index_value <= max_value:
                    return value["color"]

        return self.custom_color_for_rest

    def get_style_as_json(self):
        palette = self.get_palette_list()
        style = {
            "bands": [
                {
                    "band": 1,
                    "min": self.min,
                    "max": self.max,
                    "palette": palette,
                    "scheme": "discrete",
                }
            ]
        }
        return style

    def get_legend_config(self):
        items = []
        if self.use_custom_colors:
            values = self.get_custom_color_values()
            count = len(values)
            if count > 1:
                for value in values:
                    item = {
                        "name": value['label'] if value.get('label') else value['threshold'],
                        "color": value['color']
                    }
                    items.append(item)
                rest_item = {"name": "", "color": self.custom_color_for_rest}
                items.append(rest_item)

        return {"type": "choropleth", "items": items}


class ColorValue(TimeStampedModel, Orderable):
    layer = ParentalKey(RasterStyle, related_name='color_values')
    threshold = models.FloatField(verbose_name=_("Threshold value"), help_text=_(
        "Values less than or equal to the input value, will be assigned the chosen color"))
    color = ColorField(default="#ff0000", verbose_name=_("color"))
    label = models.CharField(max_length=100, blank=True, null=True, verbose_name=_('Optional Label'))

    class Meta:
        verbose_name = _("Color Value")
        verbose_name_plural = _("Color Values")

    panels = [
        FieldPanel("threshold"),
        NativeColorPanel("color"),
        FieldPanel("label")
    ]

    @property
    def value(self):
        return {
            "threshold": self.threshold,
            "color": self.color,
            "label": self.label
        }


@receiver(post_save, sender=RasterFileLayer)
def create_auto_ingest_directory(sender, instance, created, **kwargs):
    if instance.auto_ingest_from_directory:
        auto_ingest_raster_data_dir = geomanager_settings.get("auto_ingest_raster_data_dir")

        if auto_ingest_raster_data_dir and os.path.isabs(auto_ingest_raster_data_dir):
            if not os.path.exists(auto_ingest_raster_data_dir):
                os.makedirs(auto_ingest_raster_data_dir)

            dir_name = str(instance.pk)
            directory_path = os.path.join(auto_ingest_raster_data_dir, dir_name)

            # Create the directory if it doesn't exist
            if not os.path.exists(directory_path):
                os.makedirs(directory_path)



# Generated by Django 4.1.10 on 2023-09-27 12:31

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields
import uuid
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0019_wmslayer_get_capabilities_layer_name_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='TmsLayer',
            fields=[
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='layer title', max_length=255, verbose_name='title')),
                ('default', models.BooleanField(default=False, help_text='Is Default Layer', verbose_name='default')),
                ('base_url', models.CharField(max_length=500, verbose_name='Base url for TMS')),
                ('query_params_static', wagtail.fields.StreamField([('param', wagtail.blocks.StructBlock([('key', wagtail.blocks.CharBlock(label='Key')), ('value', wagtail.blocks.CharBlock(label='Value'))], label='Query Parameter'))], blank=True, help_text='Static query params to be added to the url', null=True, use_json_field=True, verbose_name='Static Query Params')),
                ('query_params_selectable', wagtail.fields.StreamField([('param', wagtail.blocks.StructBlock([('name', wagtail.blocks.CharBlock(label='name')), ('label', wagtail.blocks.CharBlock(label='label', required=False)), ('type', wagtail.blocks.ChoiceBlock(choices=[('radio', 'Radio'), ('dropdown', 'Dropdown')], label='Selector Type')), ('options', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('label', wagtail.blocks.CharBlock(label='label')), ('value', wagtail.blocks.CharBlock(label='value')), ('default', wagtail.blocks.BooleanBlock(help_text='Check to make default option', label='default', required=False))]), label='Options', min_num=1))], label='Selectable Query Parameter'))], blank=True, help_text='This should provide a list of options that users can choose to change the query parameter of the url', null=True, use_json_field=True, verbose_name='Query Params With selectable Options')),
                ('params_selectors_side_by_side', models.BooleanField(default=False, verbose_name='Arrange Param Selectors side by side')),
                ('legend', wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Custom Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Custom Image'))], blank=True, null=True, use_json_field=True, verbose_name='Legend')),
                ('more_info', wagtail.fields.StreamField([('more_info', wagtail.blocks.StructBlock([('link_text', wagtail.blocks.CharBlock(label='Link text', required=True)), ('link_url', wagtail.blocks.URLBlock(label='Link Url', required=True)), ('text', wagtail.blocks.TextBlock(label='Short description text', required=False)), ('is_button', wagtail.blocks.BooleanBlock(default=True, label='Format link as action button', required=False)), ('show_arrow', wagtail.blocks.BooleanBlock(default=True, label='Show arrow in button', required=False))], label='Info link'))], blank=True, null=True, use_json_field=True, verbose_name='More Info')),
                ('dataset', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tms_layers', to='geomanager.dataset', verbose_name='dataset')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
    ]



from adminboundarymanager.models import AdminBoundarySettings
from wagtail import hooks
from wagtail.api.v2.utils import get_full_url
from wagtail_modeladmin.options import modeladmin_register

from .admin import GeoManagerAdminGroup, urls as geomanager_urls
from .utils.boundary import create_boundary_dataset


@hooks.register('register_admin_urls')
def urlconf_geomanager():
    return geomanager_urls


modeladmin_register(GeoManagerAdminGroup)


@hooks.register("register_icons")
def register_icons(icons):
    return icons + [
        'wagtailfontawesomesvg/solid/palette.svg',
        'wagtailfontawesomesvg/solid/database.svg',
        'wagtailfontawesomesvg/solid/layer-group.svg',
        'wagtailfontawesomesvg/solid/globe.svg',
        'wagtailfontawesomesvg/solid/map.svg',
    ]


@hooks.register('construct_settings_menu')
def hide_settings_menu_item(request, menu_items):
    hidden_settings = ["admin-boundary-settings", "geomanager-settings"]
    menu_items[:] = [item for item in menu_items if item.name not in hidden_settings]


@hooks.register('register_geomanager_datasets')
def register_geomanager_datasets(request):
    datasets = []

    abm_settings = AdminBoundarySettings.for_request(request)

    boundary_tiles_url = abm_settings.boundary_tiles_url
    boundary_tiles_url = get_full_url(request, boundary_tiles_url)

    # create boundary dataset
    boundary_dataset = create_boundary_dataset(boundary_tiles_url)

    datasets.append(boundary_dataset)

    return datasets



from django.forms.widgets import Input


class RasterStyleWidget(Input):
    input_type = "hidden"
    template_name = "geomanager/widgets/raster_style_widget.html"

    class Media:
        js = ('geomanager/js/vendor/d3-format.min.js', 'geomanager/js/colorbrewer.js',
              'geomanager/js/widgets/raster_style_widget.js',)
